/*********************************************************************************
 *   Kenya - an introductory programming language and development environment    *
 *   Copyright (C) 2001 Robert Chatley                                           *
 *                                                                               *
 *   This program is free software; you can redistribute it and/or               *
 *   modify it under the terms of the GNU General Public License                 *
 *   as published by the Free Software Foundation; either version 2              *
 *   of the License, or (at your option) any later version.                      *
 *                                                                               *
 *   This program is distributed in the hope that it will be useful,             *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *   GNU General Public License for more details.                                *
 *                                                                               *
 *   You should have received a copy of the GNU General Public License           *
 *   along with this program; if not, write to the Free Software                 *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
 *                                                                               *
 *   The author can be contacted by email at robert@chatley.com                  *
 *                                                                               *
 *********************************************************************************/

// >>> askc98 2002/2
package com.chatley.kenya;
// <<< askc98

import com.chatley.kenya.minijava.node.*;
import com.chatley.kenya.minijava.analysis.*;
import com.chatley.kenya.utils.*;
import java.util.HashMap;

public class ClassFinder extends DepthFirstAdapter {

    private HashMap classes;
    private SymbolTable functions;

    public ClassFinder() {
        classes = new HashMap();
        functions = new SymbolTable();
    }

    public HashMap getClasses() {
        return classes;
    }

    public SymbolTable getFunctions() {
        return functions;
    }

    public void caseAClassDecDeclaration( AClassDecDeclaration node ) {

        if ( node.getIdentifier() != null ) {
            String classname = node.getIdentifier().toString().trim();
            SymbolTable fields = new SymbolTable(classname);
            fields.incompleteClass = true;
            fields.isClass = true;
            classes.put( classname , fields);
        }
    }


    public void caseAFuncDecDeclaration( AFuncDecDeclaration node ) {

        if ( node.getIdentifier() != null ) {

            String name = node.getIdentifier().toString().trim();
            String type = node.getType().toString().trim();

            if ( !functions.containsKey( name ) ) {
                SymbolTable entry = new SymbolTable( type );
                entry.prototypeOnly = true;
                if ( node.getBracketPair().size() > 0 ) {
                    entry.arraydim = node.getBracketPair().size();
                }

                // parse the parameters...
                FuncInfo f = new FuncInfo(name,type,node);
                if(node.getFormalParamList() != null)
                {

                   AFormalParamList params = (AFormalParamList)node
                     .getFormalParamList();

                   ATypeName t = (ATypeName)params.getTypeName();
                   String aType = ASTUtils.parseTypeName(t);
                   String aName = t.getIdentifier().toString().trim();
                   f.addParam(aName,aType);

                   Object temp[] = params.getCommaTypeName().toArray();
                   for(int i = 0; i < temp.length; i++)
                   {
                       t = (ATypeName) ((ACommaTypeName) temp[i]).getTypeName();
                       aType = ASTUtils.parseTypeName(t);
                       aName = t.getIdentifier().toString().trim();
                       f.addParam(aName,aType);
                   }
                }
                entry.funcInfo = f;
                functions.put( name , entry );
            }
        }
    }
}
