/* *******************************************************************************
 *   Kenya - an introductory programming language and development environment    *
 *   Copyright (C) 2001,2002 Robert Chatley, Alan Chan                           *
 *                                                                               *
 *   This program is free software; you can redistribute it and/or               *
 *   modify it under the terms of the GNU General Public License                 *
 *   as published by the Free Software Foundation; either version 2              *
 *   of the License, or (at your option) any later version.                      *
 *                                                                               *
 *   This program is distributed in the hope that it will be useful,             *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *   GNU General Public License for more details.                                *
 *                                                                               *
 *   You should have received a copy of the GNU General Public License           *
 *   along with this program; if not, write to the Free Software                 *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
 *                                                                               *
 *   The author can be contacted by email at robert@chatley.com                  *
 *                                                                               *
 *********************************************************************************/


package com.chatley.kenya;

/**
 * Interface for Kenya Type checker<p>
 *
 * This class should be used to run the kenya type checker, simply
 * create an instance of this class, and then call <code>execute()</code>
 * method to perform type checking.<p>See <code>ParseListener</code> for
 * example of use.
 *
 * @see TypeChecker
 *
 * @author Alan Chan
 */

import java.io.*;
import com.chatley.kenya.minijava.node.*;

public class KTypeChecker
{
    /** The AST which we shall be type checking.*/
    private Start mAst;

    /**
     * The destination of all error messages, all error msgs should be written
     * here.
     */
    private ErrorMsgWriter mStdErr = null;

    /** The typechecker engine that will do the type engine.*/
    private TypeChecker mTC = null;

    /**
     * Create a new <code>KTypeChecker</code> object.&nbsp;Call the
     * <code>execute()</code> method to perform the typechecker after calling
     * this constructor.
     *
     * @param errorDest writer for the typechecker to write error messages to.
     */
    public KTypeChecker(ErrorMsgWriter errorDest,Start ast)
    {
        mStdErr = errorDest;
        mAst    = ast;
    }

    /**
     * executes the typechecker on the passed Kenya program.
     *
     * @returns true if the kenya program type checks, false otherwise.
     */
    public boolean execute()
    {
        ClassFinder cf = new ClassFinder();

        mAst.apply(cf);

        mTC = new TypeChecker(mStdErr,cf.getClasses(),
          cf.getFunctions());

        mAst.apply(mTC);     // first pass...
        mTC.setSecondPass(); // prepare typechecker engine for 2nd pass...
        mAst.apply(mTC);     // second pass...

        return mTC.genCode();
    }

    /**
     * returns whether the program typechecked correctly. If this method is
     * called before the <code>execute()</code> method is called, false is
     * returned.
     */
    public boolean typeCheckOK()
    {
        return (mTC==null) ? false : mTC.genCode();
    }

    /**
     * returns the internal typechecker engine to which this is the
     * interface of.
     */
    public TypeChecker getTypeChecker()
    {
        return mTC;
    }

}