/* *******************************************************************************
 *   Kenya - an introductory programming language and development environment    *
 *   Copyright (C) 2001,2002 Robert Chatley, Alan Chan                           *
 *                                                                               *
 *   This program is free software; you can redistribute it and/or               *
 *   modify it under the terms of the GNU General Public License                 *
 *   as published by the Free Software Foundation; either version 2              *
 *   of the License, or (at your option) any later version.                      *
 *                                                                               *
 *   This program is distributed in the hope that it will be useful,             *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *   GNU General Public License for more details.                                *
 *                                                                               *
 *   You should have received a copy of the GNU General Public License           *
 *   along with this program; if not, write to the Free Software                 *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
 *                                                                               *
 *   The author can be contacted by email at robert@chatley.com                  *
 *                                                                               *
 *********************************************************************************/

/*
 * PreLoaderThread.java
 *
 * This class simply preloads and runs the parser and translator classes
 * in a separate thread with an empty input. If we run this when we start
 * Kenya for the first time, when we come to run the translator, interpreter,
 * etc., they will run without a long a wait.
 *
 * askc98 2002/5
 */
package com.chatley.kenya;

import com.chatley.kenya.interpreter.*;
import com.chatley.kenya.utils.IMTrace;
import com.chatley.kenya.minijava.node.*;
import com.chatley.kenya.minijava.lexer.*;
import com.chatley.kenya.minijava.parser.*;
import java.io.*;

class PreLoaderThread extends Thread
{
    public void run()
    {
        try{
            IMTrace.out("Starting loader thread");
            Lexer lexer = new Lexer( new PushbackReader( new StringReader("")));
            Parser parser = new Parser(lexer);
            Start ast = parser.parse();

            ClassFinder cf = new ClassFinder();
            ast.apply(cf);

            TypeChecker tc = new TypeChecker( null , cf.getClasses() , cf.getFunctions() );
            ast.apply(tc);

            DataFlowAnalyser dfa = new DataFlowAnalyser(null);
            ast.apply(dfa);

            Translator t = new Translator(tc);
            ast.apply(t);

            IMTrace.out("Loader thread done");
        }catch(Exception e){e.printStackTrace();}
    }
}