/*********************************************************************************
 *   Kenya - an introductory programming language and development environment    *
 *   Copyright (C) 2001 Robert Chatley                                           *
 *                                                                               *
 *   This program is free software; you can redistribute it and/or               *
 *   modify it under the terms of the GNU General Public License                 *
 *   as published by the Free Software Foundation; either version 2              *
 *   of the License, or (at your option) any later version.                      *
 *                                                                               *
 *   This program is distributed in the hope that it will be useful,             *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *   GNU General Public License for more details.                                *
 *                                                                               *
 *   You should have received a copy of the GNU General Public License           *
 *   along with this program; if not, write to the Free Software                 *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
 *                                                                               *
 *   The author can be contacted by email at robert@chatley.com                  *
 *                                                                               *
 *********************************************************************************/

// >>> askc98 2002/2
package com.chatley.kenya;
// <<< askc98

import com.chatley.kenya.minijava.analysis.*;
import com.chatley.kenya.minijava.node.*;

import java.util.*;
import java.io.*;

public class Translator extends DepthFirstAdapter
{

    private boolean inClass, inPrint;
    private String currentfunc;
    private Vector code;
    private HashSet imports;
    private boolean reads;
    private HashSet collections;
    private HashSet constants;

    private int indent;
    //private boolean lastTypeWasClassType, arrayDec;
    private TypeChecker tc;

    private boolean usingGraphics;
    private boolean genClassFuncDecs;
    private boolean genConsts;

    private Vector classes;
    private Vector functions;

    public Translator()
    {
        inClass = false;
        inPrint = false;
        code = new Vector();
        imports = new HashSet();
        constants = new HashSet();
        indent = 0;
        //lastTypeWasClassType = false;
        //arrayDec = false;
        usingGraphics = false;
        currentfunc = "";
        reads = false;
        collections = new HashSet();
        collections.add("Vector");
        collections.add("Stack");
        collections.add("Dictionary");
        collections.add("Hashtable");
        collections.add("Enumeration");

        classes = new Vector();
        functions = new Vector();
        genClassFuncDecs = false;
        genConsts = false;
    }

    public Translator( TypeChecker t )
    {
    this();
    tc = t;
    }
    public void newline()
    {
    addLine( "" , true);
    }

    public void newline( int a )
    {
    for ( int i = 0 ; i < a ; i++ )
        newline();
    }

    public void openbrace()
    {
    addLine("{");
    indent += 1;
    }

    public void closebrace()
    {
    indent -= 1;
    addLine("}");
    }


    public void addLine( String s )
    {
    code.addElement( new CodeLine(s , indent) );
    }

    public void addLine( String s , boolean b )
    {
    code.addElement( new CodeLine(s, indent, b));
    }

    /*public void closeClass()
    {
    indent = 0;
    addLine("");
    addLine("}");
    newline();
    addLine("");
    newline();
    }*/

    public Vector getCode()
    {
    return code;
    }

    public Iterator getImports()
    {
    return imports.iterator();
    }

    public boolean getReads()
    {
    return reads;
    }

    public Iterator getConstants()
    {
        return constants.iterator();
    }

    private boolean isPrimitiveType(String type)
    {
        if(type.equals("int")) return true;
        else if(type.equals("double")) return true;
        else if(type.equals("char")) return true;
        else if(type.equals("boolean")) return true;
        else if(type.equals("String")) return true;
        else return false;
    }

    public void inStart(Start node)
    {
        indent = 1;
        newline();
        //addLine("public static void main(String[] args) ");
        //openbrace();
        //newline();
        //addLine("");
        //newline();
        addLine("graphics");
    }

    public void outStart(Start node)
    {
        //indent--;
        //closebrace();
        indent = 1;
        //newline();
        //newline();
        indent = 0;
    }


    // ************ Declarations ************ //

    public void caseAClassDecDeclaration(AClassDecDeclaration node)
    {
        inClass = true;
        newline();

        if ( /*ParseListener.tree */!genClassFuncDecs)
        {
            //ParseListener.classes.add(node);
            classes.add(node);
        }

        else
        {
            newline();
            addLine("class ");

            if ( node.getIdentifier() != null )
            {
                node.getIdentifier().apply(this);
            }

            newline();
            openbrace();
            newline();

            LinkedList decs = node.getDeclaration();
            ListIterator decsIterator = decs.listIterator(0);

            while( decsIterator.hasNext() )
            {
                Node t = (Node)decsIterator.next();
                t.apply(this);
            }

            closebrace();
        }

        inClass = false;
    }

    public void caseAFuncDecDeclaration(AFuncDecDeclaration node)
    {

        currentfunc = node.getIdentifier().toString().trim();

        // we process functions at the end, apart from main(), which
        // we put towards the top of the class.
    if (!genClassFuncDecs && !inClass && !"main".equals(currentfunc))
    {
            functions.add(node);
    }
    else
    {
        if ( !inClass )
        {
            indent = 1;
            if(!"main".equals(currentfunc)) addLine("  static ");
            else addLine("public static ");
        }
        else
        {
            newline();
        }

        if ( node.getType() != null ) {
            node.getType().apply(this);
        }

        if ( node.getBracketPair() != null ) {

        LinkedList bps = node.getBracketPair();

        for( ListIterator bpIter = bps.listIterator(0); bpIter.hasNext() ; bpIter.next() )
            addLine("[]");
        }

	if ( !"main".equals(currentfunc) ) {
	    if ( node.getType() != null && ! isPrimitiveType( node.getType().toString().trim() ) ) {
		addLine(" ");
	    }
	}      

        if ( node.getIdentifier() != null )
        {
            node.getIdentifier().apply(this);
            //currentfunc = node.getIdentifier().toString().trim();
        }

        addLine("( ");

        if("main".equals(currentfunc))
        {
            addLine("String[] args");
        }
        else if ( node.getFormalParamList() != null )
        {
            node.getFormalParamList().apply(this);
        }

        addLine(" )");
     
	addLine(" ");
	//newline();

        if ( node.getBlock() != null )
        {
            node.getBlock().apply(this);
        }

        newline();
    }

    currentfunc = "";
    }

    public void caseAFormalParamList(AFormalParamList node) {

    if ( node.getTypeName() != null ) {
        node.getTypeName().apply(this);
    }

    LinkedList ctns = node.getCommaTypeName();
        ListIterator ctnsIterator = ctns.listIterator(0);

    while( ctnsIterator.hasNext() ) {

        Node t = (Node)ctnsIterator.next();
            t.apply(this);
        }
    }

    public void caseATypeName(ATypeName node)
    {
    if ( node.getType() != null )
    {
        addLine( node.getType().toString().trim() + " " );
    }

    if ( node.getBracketPair() != null ) {

        LinkedList bps = node.getBracketPair();

        for ( ListIterator bpIter = bps.listIterator(0) ; bpIter.hasNext() ; bpIter.next() )
        addLine("[]");
    }

    if ( node.getIdentifier() != null )
    {
        node.getIdentifier().apply(this);
    }
    }

    public void caseACommaTypeName(ACommaTypeName node)
    {
    addLine(" , ");

    if ( node.getTypeName() != null )
    {
        node.getTypeName().apply(this);
    }
    }

    public void caseAConstDecDeclaration(AConstDecDeclaration node)
    {
    if ( /*ParseListener.tree*/ !genConsts && !inClass )
        {
            constants.add(node);
    }
    else
    {
        addLine("  static final ");

        if ( node.getType() != null )
        {
        node.getType().apply(this);
        }

        if ( node.getIdentifier() != null )
        {
        node.getIdentifier().apply(this);
        }

        if ( node.getInitialiser() != null )
        {
        node.getInitialiser().apply(this);
        }

        addLine(";");
        newline(2);
    }
    }

    public void caseAVarDecDeclaration(AVarDecDeclaration node)
    {
        boolean lastTypeWasClassType = false;

        if ( node.getType() != null )
        {
            node.getType().apply(this);
        }

        String type = node.getType().toString().trim();

        if(!isPrimitiveType(type)){
            lastTypeWasClassType = true;
            addLine(" ");
        }

    /*if ( node.getBracketPair().size() > 0 ) {

        LinkedList bps = node.getBracketPair();
        ListIterator bpIterator = bps.listIterator(0);

        while( bpIterator.hasNext() ) {
        bpIterator.next();
        addLine("[ ]");
        arrayDec = true;
        }
    }

    else if ( node.getArrayAccess().size() > 0 ) {

        LinkedList accesses = node.getArrayAccess();
        ListIterator aIterator = accesses.listIterator(0);

        while( aIterator.hasNext() ) {
        aIterator.next();
        addLine("[ ]");
        arrayDec = true;
        }
    }*/

        //if ( lastTypeWasClassType /*|| arrayDec */)

        if ( node.getTypeParam() != null )
        {
            node.getTypeParam().apply(this);
        }

        if ( node.getIdentifier() != null )
        {
            node.getIdentifier().apply(this);
        }

        if ( lastTypeWasClassType && !inClass /*&& !arrayDec*/ && node.getInitialiser() == null )
        {
            addLine(" = new ");
            node.getType().apply(this);

            if ( node.getTypeParam() != null )
            {
                node.getTypeParam().apply(this);
            }
            addLine("()");
            lastTypeWasClassType = false;
        }


        /*if ( arrayDec && node.getInitialiser() == null )
        {
            addLine(" = new ");
            node.getType().apply(this);

            LinkedList accesses = node.getArrayAccess();
            ListIterator aIterator = accesses.listIterator(0);

            while( aIterator.hasNext() ) {
                Node t = (Node)aIterator.next();
                t.apply(this);
            }

            arrayDec = false;
            if(lastTypeWasClassType) lastTypeWasClassType = false;
        }*/

        if ( node.getInitialiser() != null )
        {
            node.getInitialiser().apply(this);
            //arrayDec = false;
        }

        addLine(";");
        newline();
        lastTypeWasClassType = false;
    }

    public void caseAArrayDecDeclaration(AArrayDecDeclaration node)
    {
        if(node.getType()!=null) node.getType().apply(this);

        Object temp[] = node.getBracketPair().toArray();
        for(int i = 0; i < temp.length; i++)
        {
            addLine("[]");
        }

        addLine(" ");
        if(node.getIdentifier()!=null) node.getIdentifier().apply(this);
        if(node.getArrayInitialiser()!=null)
            node.getArrayInitialiser().apply(this);

        addLine(";");
        newline();
    }

    public void caseAInitialiser( AInitialiser node )
    {
        addLine(" = ");
        if ( node.getExpression() != null )
        {
            node.getExpression().apply(this);
        }
    }

    /*public void caseAArrayInitialiser( AArrayInitialiser node )
    {
    addLine( " = " );
    if ( node.getArrayInit() != null )
        node.getArrayInit().apply( this );
    }*/

    public void caseAStaticArrayInitialiser(AStaticArrayInitialiser node)
    {
        if ( node.getArrayInit() != null ){
            addLine( " = " );
            node.getArrayInit().apply( this );
        }
    }

    public void caseADynamicArrayInitialiser(ADynamicArrayInitialiser node)
    {
        if(node.getArrayAllocate()!=null){
            addLine(" = ");
            node.getArrayAllocate().apply(this);
        }
    }

    public void caseAArrayInit( AArrayInit node )
    {
    addLine("{ ");

    if ( node.getInitList() != null )
        node.getInitList().apply( this );

        addLine(" }");
    }

    public void caseAScalarInitList( AScalarInitList node )
    {
    if ( node.getExpression() != null )
        node.getExpression().apply( this );

    if ( node.getCommaExp() != null ) {

        LinkedList commaexps = node.getCommaExp();
        ListIterator ceIterator = commaexps.listIterator(0);

        while( ceIterator.hasNext() )
        ((Node)ceIterator.next()).apply( this );
    }
    }

    public void caseACommaExp( ACommaExp node )
    {
    addLine( " , " );

    if ( node.getExpression() != null )
        node.getExpression().apply( this );
    }

    public void caseAArrayInitList( AArrayInitList node )
    {
    if ( node.getArrayInit() != null )
        node.getArrayInit().apply( this );

    if ( node.getCommaArrayInit() != null ) {

        LinkedList commaarrayinits = node.getCommaArrayInit();
        ListIterator caiIterator = commaarrayinits.listIterator(0);

        while( caiIterator.hasNext() )
        ((Node)caiIterator.next()).apply( this );

    }
    }

    public void caseACommaArrayInit( ACommaArrayInit node )
    {
    addLine( " , " );

    if ( node.getArrayInit() != null )
        node.getArrayInit().apply( this );
    }

    public void caseAClassType( AClassType node)
    {
        //lastTypeWasClassType = true;

        /*if ( node.getName() != null )
        {
            PName t = node.getName();

            if ( collections.contains( t.toString().trim() ) )
            {
               imports.add( t.toString() );
            }

            t.apply(this);
        }*/

        if ( node.getIdentifier() != null )
        {
            TIdentifier t = node.getIdentifier();

            if ( collections.contains( t.toString().trim() ) )
            {
               imports.add( t.toString() );
            }

            t.apply(this);
        }
    }

   public void caseATypeParamList( ATypeParamList node )
    {
    addLine("< ");

    if ( node.getType() != null )
    {
        node.getType().apply(this);
    }

    LinkedList params = node.getCommaType();
    ListIterator paramsIterator = params.listIterator(0);

    while( paramsIterator.hasNext() )
    {
        ACommaType t = (ACommaType)paramsIterator.next();
        addLine(", ");
        t.getType().apply(this);
    }

    addLine(" > ");
    }

    public void caseATypeParam( ATypeParam node )
    {
    if ( node.getTypeParamList() != null )
    {
        node.getTypeParamList().apply(this);
    }
    }

    public void caseAArrayAccess( AArrayAccess node ) {

    addLine("[");

    if ( node.getMathExpression() != null ) {
        node.getMathExpression().apply(this);
    }

    addLine("]");
    }

    public void caseAQualifiedName( AQualifiedName node )
    {
    if ( node.getFieldAccess() != null )
    {
        node.getFieldAccess().apply(this);
    }

    addLine(".");

    if (node.getIdentifier() != null)
    {
        node.getIdentifier().apply(this);
    }
    }

    public void caseASimpleName( ASimpleName node )
    {
    if ( node.getIdentifier() != null )
        {
        node.getIdentifier().apply(this);
        }
    }

   public void caseASimpleNameName( ASimpleNameName node )
    {
    if ( node.getSimpleName() != null )
        {
        node.getSimpleName().apply(this);
        }
    }

    public void caseTIdentifier ( TIdentifier node )
    {
    if (node.getText() != null)
    {
        addLine(node.getText());
    }
    }

    public void caseACharBasicType(ACharBasicType node)
    {
    addLine("char ");
    }

    public void caseAIntBasicType(AIntBasicType node)
    {
    addLine("int ");
    }

    public void caseABooleanBasicType(ABooleanBasicType node)
    {
    addLine("boolean ");
    }

    public void caseARealBasicType(ARealBasicType node)
    {
        addLine("double ");
    }

    public void caseAStringBasicType(AStringBasicType node)
    {
    addLine("String ");
    }

    public void caseAVoidBasicType(AVoidBasicType node)
    {
    addLine("void ");
    }


    // ************* Expressions ***************** //

    public void caseAArrayAllocate(AArrayAllocate node)
    {
        addLine(" new ");
        if(node.getType()!=null) node.getType().apply(this);

        Object temp[] = node.getArrayAccess().toArray();
        for(int i = 0; i < temp.length; i++)
        {
            ((PArrayAccess) temp[i]).apply(this);
        }

        temp = node.getBracketPair().toArray();
        for(int i = 0; i < temp.length; i++)
        {
            addLine("[]");
        }
    }

    public void caseAOrBoolExpression( AOrBoolExpression node )
    {
    if ( node.getBoolExpression() != null ) {
        node.getBoolExpression().apply(this);
    }

    addLine(" || ");

    if ( node.getBoolTerm() != null ) {
        node.getBoolTerm().apply(this);
    }
    }

    public void caseAXorBoolExpression( AXorBoolExpression node )
    {
    if ( node.getBoolExpression() != null ) {
        node.getBoolExpression().apply(this);
    }

    addLine(" ^ ");

    if ( node.getBoolTerm() != null ) {
        node.getBoolTerm().apply(this);
    }
    }

    public void caseAAndBoolTerm( AAndBoolTerm node )
    {
    if ( node.getBoolTerm() != null ) {
        node.getBoolTerm().apply(this);
    }

    addLine(" && ");

    if ( node.getEquality() != null ) {
        node.getEquality().apply(this);
    }
    }

    public void caseAEqEquality( AEqEquality node )
    {
    TypeChecker foreq;

    if ( currentfunc.equals( "" ) ) {
        foreq = tc;
    } else {
        foreq = new TypeChecker( tc.mErrorWriter , tc.knownTypes , (SymbolTable)tc.symbolTable.get( currentfunc ) );
    }

    foreq.checked = true;
    foreq.expected = "String";

    if ( node.getEquality() != null ) {
        node.getEquality().apply( foreq );
        node.getEquality().apply( this );
    }

    boolean stringcomp = foreq.found.trim().equals( "String" );

    if ( stringcomp ) {
        addLine( ".equals( " );
    } else {
        addLine( " == " );
    }

    if ( node.getRelational() != null ) {
        node.getRelational().apply( this );
    }

    if ( stringcomp )
        addLine( " )" );
    }

    public void caseANeqEquality( ANeqEquality node )
    {
    TypeChecker foreq;

    if ( currentfunc.equals( "" ) ) {
        foreq = tc;
    } else {
        foreq = new TypeChecker( tc.mErrorWriter , tc.knownTypes , (SymbolTable)tc.symbolTable.get( currentfunc ) );
    }

    foreq.checked = true;
    foreq.expected = "String";

    if ( node.getEquality() != null ) {
        node.getEquality().apply( foreq );
    }

    if ( foreq.checked )
        addLine( " ! ( " );

    node.getEquality().apply( this );

    if ( foreq.checked ) {
        addLine( ".equals( ");
    } else {
        addLine( " != " );
    }

    if ( node.getRelational() != null )
        node.getRelational().apply( this );

    if ( foreq.checked )
        addLine( " ) )" );
    }

    public void caseALtRelational( ALtRelational node )
    {
    TypeChecker forrel;

    if ( currentfunc.equals( "" ) ) {
        forrel = tc;
    } else {
        forrel = new TypeChecker( tc.mErrorWriter , tc.knownTypes , (SymbolTable)tc.symbolTable.get( currentfunc ) );
    }

    forrel.checked = true;
    forrel.expected = "String";

    if ( node.getRelational() != null ) {
        node.getRelational().apply( forrel );
        node.getRelational().apply( this );
    }

    boolean stringcomp = forrel.found.trim().equals( "String" );

    if ( stringcomp ) {
        addLine( ".compareTo( " );
    } else {
        addLine(" < " );
    }

    if ( node.getMathExpression() != null ) {
        node.getMathExpression().apply( this );
    }

    if ( stringcomp ) {
        addLine( ") < 0 ");
    }
    }

    public void caseAGtRelational( AGtRelational node )
    {
    TypeChecker forrel;

    if ( currentfunc.equals( "" ) ) {
        forrel= tc;
    } else {
        forrel = new TypeChecker( tc.mErrorWriter , tc.knownTypes , (SymbolTable)tc.symbolTable.get( currentfunc ) );
    }

    forrel.checked = true;
    forrel.expected = "String";

    

    if ( node.getRelational() != null ) {
        node.getRelational().apply( forrel );
        node.getRelational().apply( this );
    }

    boolean stringcomp = forrel.found.trim().equals( "String" );

    if ( stringcomp ) {
        addLine( ".compareTo( " );
    } else {
        addLine(" > " );
    }

    if ( node.getMathExpression() != null ) {
        node.getMathExpression().apply( this );
    }

    if ( stringcomp ) {
        addLine( ") > 0 ");
    }
    }

    public void caseALteqRelational( ALteqRelational node )
    {
    TypeChecker forrel;

    if ( currentfunc.equals( "" ) ) {
        forrel = tc;
    } else {
        forrel = new TypeChecker( tc.mErrorWriter , tc.knownTypes , (SymbolTable)tc.symbolTable.get( currentfunc ) );
    }

    forrel.checked = true;
    forrel.expected = "String";

    if ( node.getRelational() != null ) {
        node.getRelational().apply( forrel );
        node.getRelational().apply( this );
    }

    boolean stringcomp = forrel.found.trim().equals( "String" );

    if ( stringcomp ) {
        addLine( ".compareTo( " );
    } else {
        addLine(" <= " );
    }

    if ( node.getMathExpression() != null ) {
        node.getMathExpression().apply( this );
    }

    if ( stringcomp ) {
        addLine( ") <= 0 ");
    }
    }

    public void caseAGteqRelational( AGteqRelational node )
    {
    TypeChecker forrel;

    if ( currentfunc.equals( "" ) ) {
        forrel = tc;
    } else {
        forrel = new TypeChecker( tc.mErrorWriter , tc.knownTypes , (SymbolTable)tc.symbolTable.get( currentfunc ) );
    }

    forrel.checked = true;
    forrel.expected = "String";

    if ( node.getRelational() != null ) {
        node.getRelational().apply( forrel );
        node.getRelational().apply( this );
    }

    boolean stringcomp = forrel.found.trim().equals( "String" );

    if ( stringcomp ) {
        addLine( ".compareTo( " );
    } else {
        addLine(" >= " );
    }

    if ( node.getMathExpression() != null ) {
        node.getMathExpression().apply( this );
    }

    if ( stringcomp ) {
        addLine( ") >= 0 ");
    }
    }

    public void caseANegateUnaryExp( ANegateUnaryExp node )
    {
    addLine("!");
    node.getUnaryExp().apply(this);
    }

    public void caseATermMathExpression( ATermMathExpression node )
    {
    if ( node.getTerm() != null )
        node.getTerm().apply(this);
    }

    public void caseAPlusMathExpression( APlusMathExpression node )
    {

    if ( ! inPrint ) {

        TypeChecker forplus;

        if ( currentfunc.equals( "" ) ) {
        forplus = tc;
        } else {
        forplus = new TypeChecker( tc.mErrorWriter , tc.knownTypes , (SymbolTable)tc.symbolTable.get( currentfunc ) );
        }

        forplus.checked = true;
        forplus.expected = "String";

        if ( node.getMathExpression() != null ) {
        node.getMathExpression().apply( forplus );
        node.getMathExpression().apply( this );
        }

        if ( forplus.checked ) {
        addLine( ".concat( " );
        } else {
        addLine(" + " );
        }

        if ( node.getTerm() != null ) {
        node.getTerm().apply( this );
        }

        if ( forplus.checked ) {
        addLine( ") ");

        }

    } else {

        if ( node.getMathExpression() != null )
        node.getMathExpression().apply(this);

        addLine(" + ");

        if ( node.getTerm() != null )
        node.getTerm().apply(this);
    }
    }

    public void caseAMinusMathExpression( AMinusMathExpression node )
    {
    if ( node.getMathExpression() != null )
        node.getMathExpression().apply(this);

    addLine(" - ");

    if ( node.getTerm() != null )
        node.getTerm().apply(this);
    }

    public void caseAUnaryTerm( AUnaryTerm node )
    {
    if ( node.getUnaryExp() != null )
        node.getUnaryExp().apply(this);
    }

    public void caseAMultTerm( AMultTerm node )
    {
    if ( node.getTerm() != null )
        node.getTerm().apply(this);

    addLine(" * ");

    if ( node.getUnaryExp() != null )
        node.getUnaryExp().apply(this);
    }

    public void caseAPowerTerm( APowerTerm node )
    {
    addLine(" Math.pow( ");

    if ( node.getTerm() != null )
        node.getTerm().apply(this);

    addLine(", ");

    if ( node.getUnaryExp() != null )
        node.getUnaryExp().apply(this);

    addLine(" )");
    }

    public void caseADivTerm( ADivTerm node )
    {
    if ( node.getTerm() != null )
        node.getTerm().apply(this);

    addLine(" / ");

    if ( node.getUnaryExp() != null )
        node.getUnaryExp().apply(this);
    }

    public void caseAModTerm( AModTerm node )
    {
    if ( node.getTerm() != null )
        node.getTerm().apply(this);

    addLine(" % ");

    if ( node.getUnaryExp() != null )
        node.getUnaryExp().apply(this);
    }

    public void caseAReadintFactor( AReadintFactor node )
    {
    reads = true;
    addLine("InputReader.readInt()");
    }

    public void caseAReadrealFactor( AReadrealFactor node )
    {
    reads = true;
    addLine("InputReader.readDouble()");
    }

    public void caseAReadstringFactor( AReadstringFactor node )
    {
    reads = true;
    addLine("InputReader.readString()");
    }

    public void caseAReadcharFactor( AReadcharFactor node )
    {
    reads = true;
    addLine("InputReader.readChar()");
    }

    public void caseAReadFactor( AReadFactor node )
    {
    reads = true;
    addLine("InputReader.read()");
    }


    public void caseAEndoffileFactor( AEndoffileFactor node )
    {
    reads = true;
    addLine("InputReader.isEOF()");
    }

    public void caseASinFactor( ASinFactor node )
    {
    addLine("Math.sin(");
    if ( node.getMathExpression() != null ) {
        node.getMathExpression().apply( this );
    }
    addLine(")");
    }

    public void caseAAsinFactor( AAsinFactor node )
    {
    addLine("Math.asin(");
    if ( node.getMathExpression() != null ) {
        node.getMathExpression().apply( this );
    }
    addLine(")");
    }

    public void caseACosFactor( ACosFactor node )
    {
    addLine("Math.cos(");
    if ( node.getMathExpression() != null ) {
        node.getMathExpression().apply( this );
    }
    addLine(")");
    }

    public void caseAAcosFactor( AAcosFactor node )
    {
    addLine("Math.acos(");
    if ( node.getMathExpression() != null ) {
        node.getMathExpression().apply( this );
    }
    addLine(")");
    }

    public void caseATanFactor( ATanFactor node )
    {
    addLine("Math.tan(");
    if ( node.getMathExpression() != null ) {
        node.getMathExpression().apply( this );
    }
    addLine(")");
    }

    public void caseAAtanFactor( AAtanFactor node )
    {
    addLine("Math.atan(");
    if ( node.getMathExpression() != null ) {
        node.getMathExpression().apply( this );
    }
    addLine(")");
    }

    public void caseALogFactor( ALogFactor node )
    {
    addLine("Math.log(");
    if ( node.getMathExpression() != null ) {
        node.getMathExpression().apply( this );
    }
    addLine(")");
    }

    public void caseASqrtFactor( ASqrtFactor node )
    {
    addLine("Math.sqrt(");
    if ( node.getMathExpression() != null ) {
        node.getMathExpression().apply( this );
    }
    addLine(")");
    }

    public void caseAExpFactor( AExpFactor node )
    {
    addLine("Math.exp(");
    if ( node.getMathExpression() != null ) {
        node.getMathExpression().apply( this );
    }
    addLine(")");
    }

    public void caseARoundFactor( ARoundFactor node )
    {
    addLine("(int)Math.round(");
    if ( node.getMathExpression() != null ) {
        node.getMathExpression().apply( this );
    }
    addLine(")");
    }

    public void caseACeilFactor( ACeilFactor node )
    {
    addLine("(int)Math.ceil(");
    if ( node.getMathExpression() != null ) {
        node.getMathExpression().apply( this );
    }
    addLine(")");
    }

    public void caseAFloorFactor( AFloorFactor node )
    {
    addLine("(int)Math.floor(");
    if ( node.getMathExpression() != null ) {
        node.getMathExpression().apply( this );
    }
    addLine(")");
    }

    public void caseARandomFactor( ARandomFactor node )
    {
    addLine("Math.random()");
    }

    public void caseAStringliteralFactor(AStringliteralFactor node)
    {
    if ( node != null )
    {
        addLine( node.getStringliteral().toString() );
    }
    }

    public void caseACharliteralFactor(ACharliteralFactor node)
    {
    if ( node != null )
    {
        addLine( node.getCharliteral().toString() );
    }
    }

    public void caseABoolliteralFactor(ABoolliteralFactor node)
    {
    if ( node != null )
    {
        addLine( node.getBooleanliteral().toString() );
    }
    }

    public void caseANumberFactor( ANumberFactor node )
    {
    if ( node.getNumber() != null )
        node.getNumber().apply(this);
    }

    public void caseANullFactor( ANullFactor node )
    {
    addLine(" null ");
    }

    public void caseAExpressionFactor( AExpressionFactor node )
    {
    addLine("( ");

    if ( node.getBoolExpression() != null )
        node.getBoolExpression().apply(this);

    addLine(" )");
    }

    public void caseAPreincr( APreincr node )
    {
    addLine("++");
    if (node.getPostfixExp() != null )
        node.getPostfixExp().apply( this );
    }

    public void caseAPredecr( APredecr node )
    {
    addLine("--");
    if (node.getPostfixExp() != null )
        node.getPostfixExp().apply( this );
    }

    public void caseAPostincr( APostincr node )
    {
    if (node.getPostfixExp() != null )
        node.getPostfixExp().apply( this );
    addLine("++");
    }

    public void caseAPostdecr( APostdecr node )
    {
    if (node.getPostfixExp() != null )
        node.getPostfixExp().apply( this );
    addLine("--");
    }

    public void caseAMinusUnaryExp( AMinusUnaryExp node )
    {
    addLine( "-" );
    if ( node.getUnaryExp() != null )
        node.getUnaryExp().apply( this );
    }

    public void caseAPlusUnaryExp( APlusUnaryExp node )
    {
    addLine( "+" );
    if ( node.getUnaryExp() != null )
        node.getUnaryExp().apply( this );
    }

    public void caseAFNumber( AFNumber node )
    {
    addLine( node.toString().trim() );
    }

    public void caseAINumber( AINumber node )
    {
    addLine( node.toString().trim() );
    }

    public void caseAAssignmentStatement(AAssignmentStatement node)
    {
    if ( node.getFieldAccess() != null )
    {
        node.getFieldAccess().apply(this);
    }

    addLine(" = ");

    if ( node.getExpression() != null )
    {
        node.getExpression().apply(this);
    }

    addLine(";");
    newline();
    }

    public void caseASwitchStatement(ASwitchStatement node)
    {
    if ( node.getSwitch() != null )
    {
        addLine("switch ( ");
    }
    if (node.getExpression() != null )
    {
        node.getExpression().apply(this);
    }
    addLine(" )");
    newline();

    if ( node.getSwitchBlock() != null )
    {
        node.getSwitchBlock().apply(this);
    }
    }

    public void caseASwitchBlock(ASwitchBlock node)
    {
    openbrace();
    newline();

    if ( node.getPossibleCase() != null )
    {
        LinkedList cases = node.getPossibleCase();
        ListIterator casesIterator = cases.listIterator(0);

        while( casesIterator.hasNext() )
        {
        Node t = (Node)casesIterator.next();
        t.apply(this);
        }
    }

    closebrace();
    newline();
    }

    public void caseAPossibleCase(APossibleCase node)
    {
    if ( node.getSwitchLabel() != null )
    {
        node.getSwitchLabel().apply(this);
    }

    if ( node.getBlock() != null )
    {
        node.getBlock().apply(this);
    }
    }

    public void caseACaseSwitchLabel(ACaseSwitchLabel node)
    {
    addLine("case ");

    if ( node.getExpression() != null )
    {
        node.getExpression().apply(this);
    }
    addLine(": ");
    newline();
    }

    public void caseADefaultSwitchLabel(ADefaultSwitchLabel node)
    {
    addLine("default :");
    newline();
    }

    public void caseABreakStatement(ABreakStatement node)
    {
    addLine("break;");
    newline();
    }

    public void caseAReturnStatement(AReturnStatement node)
    {
    if ( node.getReturn() != null )
    {
        addLine("return ");
        }

    if ( node.getExpression() != null )
    {
        node.getExpression().apply( this );
        }
    addLine(";");
    newline();
    }

    public void caseAFunctioncallStatement(AFunctioncallStatement node)
    {
    if ( node.getFunctionApplication() != null )
    {
        node.getFunctionApplication().apply( this );
        }
    addLine(";");
    newline();
    }

    public void caseAFunctionApplication(AFunctionApplication node)
    {
    if ( node.getName() != null )
    {
        node.getName().apply(this);
    }

    addLine("( ");

    if ( node.getActualParamList() != null )
    {
        node.getActualParamList().apply(this);
        }

    addLine(")");
    }

    public void caseAExpActualParamList(AExpActualParamList node)
    {
    if ( node.getExpression() != null )
    {
        node.getExpression().apply(this);
    }
    }

    public void caseAListActualParamList(AListActualParamList node)
    {
    if ( node.getActualParamList() != null )
    {
        node.getActualParamList().apply(this);
    }

    if ( node.getComma() != null )
    {
        addLine(", ");
    }

    if ( node.getExpression() != null )
    {
        node.getExpression().apply(this);
    }
    }


    public void caseAIfStatement(AIfStatement node)
    {
    addLine("if ( ");

    if ( node.getBoolExpression() != null )
        {
        node.getBoolExpression().apply(this);
    }

    addLine( " )" );
    newline();

    if ( node.getBlock1() != null )
    {
        node.getBlock1().apply(this);
    }

    if ( node.getElse() != null )
    {
        addLine("else");
        newline();

        if ( node.getBlock2() != null )
        {
        node.getBlock2().apply(this);
        }
    }
    }

    public void caseADecStatement(ADecStatement node)
    {
    if ( node.getDeclaration() != null )
    {
        node.getDeclaration().apply(this);
    }
    }

    public void caseAForStatement(AForStatement node)
    {
    addLine("for");
    if ( node.getForControl() != null )
        node.getForControl().apply( this );

    if ( node.getBlock() != null )
        node.getBlock().apply( this );
    }

    public void caseASimpleForControl( ASimpleForControl node)
    {
    String index = "";
    boolean decreasing = ( node.getDecreasing() != null );

        if ( node.getName() != null )
    {
        index = node.getName().toString();
    }

    addLine("( " + index + "= ");

    if  ( node.getExp1() != null )
    {
        node.getExp1().apply(this);
    }

    if ( decreasing )
    {
        addLine("; " + index + ">= ");
    }
    else
        {
        addLine("; " + index + "<= ");
    }

    if  ( node.getExp2() != null )
    {
        node.getExp2().apply(this);
    }
    addLine("; ");

    if ( node.getStepBit() != null )
    {
        AStepBit sb = (AStepBit)node.getStepBit();

        if ( decreasing )
        {
        addLine( index + "-= ");
        }
            else
        {
        addLine( index + "+= ");
        }

        if  ( sb.getExpression() != null )
        {
        sb.getExpression().apply(this);
        }
    }
    else
    {
        if ( decreasing )
        {
        addLine( index + "-- ");
        }
        else
        {
        addLine( index + "++ ");
        }
        }

    addLine(")");
    newline();
    }

    public void caseAJavaForControl( AJavaForControl node )
    {
    addLine( "( " + node.getName().toString() + "= ");
    if ( node.getExpression() != null )
        node.getExpression().apply( this );

    addLine(" ; ");

    if ( node.getBoolExpression() != null )
        node.getBoolExpression().apply( this );

    addLine(" ; ");

    if ( node.getUnaryExp() != null )
        node.getUnaryExp().apply( this );

    addLine(")");
    newline();
    }

    public void caseAWhileStatement(AWhileStatement node)
    {
    addLine("while ( ");

    if ( node.getBoolExpression() != null )
        {
        node.getBoolExpression().apply(this);
    }

    addLine( " )" );
    newline();

    if ( node.getBlock() != null )
    {
        node.getBlock().apply(this);
    }
    }

    public void caseABlock(ABlock node)
    {
    openbrace();
    newline();

    if (node.getStatements() != null )
    {
        node.getStatements().apply(this);
    }

    closebrace();
    newline();
    }

    public void caseAPrintStrStatement(APrintStrStatement node)
    {
    inPrint = true;

        addLine("System.out.print( ");

    if ( node.getExpression() != null )
    {
        node.getExpression().apply(this);
    }

    addLine(" );");
    newline();
    inPrint = false;
    }

    public void caseAPrintlnStrStatement(APrintlnStrStatement node)
    {
    inPrint = true;

    addLine("System.out.println( ");
    if ( node.getExpression() != null )
    {
        node.getExpression().apply(this);
    }

    addLine(" );");
    newline();

    inPrint = false;
    }

    public void caseAAssertStatement( AAssertStatement node ) {

    if ( System.getProperty( "java.version" ).compareTo( "1.4" ) < 0 ) {

        addLine( "//assert is not implemented in versions of Java below 1.4");
        newline();
        addLine( "//assert " );

    } else {
        addLine("assert ");
    }

    if ( node.getBoolExpression() != null ) {
        node.getBoolExpression().apply( this );
    }

    if ( node.getColonString() != null )
        node.getColonString().apply( this );

    addLine( ";" );
    newline();
    }

    public void caseAColonString( AColonString node ) {

    addLine( " : " );
    node.getExpression().apply( this );
    }

    public void caseAArraycopyStatement( AArraycopyStatement node ) {

    addLine("System.arraycopy( ");

    node.getSrc().apply( this );
    addLine( ", 0, " );
    node.getDest().apply( this );

    addLine( " , 0 , Math.min( " );
    node.getSrc().apply( this );
    addLine( ".length, " );

    node.getDest().apply( this );
    addLine( ".length ) );" );

    newline();
    }

    /*public void caseACommentStatement( ACommentStatement node ) {

    newline(2);
    addLine( node.getComment().toString() );
    newline();
    }*/

    // Graphics primitives

    public void caseAClearStatement( AClearStatement node ) {

    usingGraphics = true;

    addLine("gw.clear();");
    newline();
    }

    public void caseADrawlineStatement( ADrawlineStatement node ) {

    usingGraphics = true;

    addLine("gw.drawLine( ");

    if ( node.getFourNumbers() != null ) {

        node.getFourNumbers().apply( this );
    }

    addLine(" );");
    newline();
    }

    public void caseADrawboxStatement( ADrawboxStatement node ) {

    usingGraphics = true;

    addLine("gw.drawBox( ");

    if ( node.getFourNumbers() != null ) {

        node.getFourNumbers().apply( this );
    }

    addLine(" );");
    newline();
    }


    public void caseADrawtriangleStatement( ADrawtriangleStatement node ) {

    usingGraphics = true;

    addLine("gw.drawTriangle( ");

    if ( node.getSixNumbers() != null ) {

        node.getSixNumbers().apply( this );
    }

    addLine(" );");
    newline();
    }



    public void caseADrawellipseStatement( ADrawellipseStatement node ) {

    usingGraphics = true;

    addLine("gw.drawEllipse( ");

    if ( node.getFourNumbers() != null ) {

        node.getFourNumbers().apply( this );
    }

    addLine(" );");
    newline();
    }

    public void caseADrawstringStatement( ADrawstringStatement node ) {

    usingGraphics = true;

    addLine("gw.drawString( ");

    if ( node.getString() != null )
        node.getString().apply( this );

    addLine(" , ");

    if ( node.getX() != null )
        node.getX().apply( this );

    addLine(" , ");

    if ( node.getY() != null )
        node.getY().apply( this );

    addLine(");");
    newline();
    }

    public void caseAFourNumbers ( AFourNumbers node ) {

    if ( node.getX1() != null ) {
        node.getX1().apply( this );
    }

    addLine( " , " );

    if ( node.getY1() != null ) {
        node.getY1().apply( this );
    }

    addLine( " , " );

    if ( node.getX2() != null ) {
        node.getX2().apply( this );
    }

    addLine( " , " );

    if ( node.getY2() != null ) {
        node.getY2().apply( this );
    }
    }

    public void caseASixNumbers ( ASixNumbers node ) {

    if ( node.getFourNumbers() != null ) {
        node.getFourNumbers().apply( this );
    }

    addLine( " , " );

    if ( node.getX() != null ) {
        node.getX().apply( this );
    }

    addLine( " , " );

    if ( node.getY() != null ) {
        node.getY().apply( this );
    }

    }

    public void caseAFillStatement( AFillStatement node ) {

    usingGraphics = true;

    addLine( "gw.fill( ");

    if ( node.getBoolExpression() != null ) {
        node.getBoolExpression().apply( this );
    }

    addLine( " );");
    newline();
    }

    public void caseASetcolourStatement( ASetcolourStatement node ) {

    addLine( "gw.setFGColour( " );

    if ( node.getRgb() != null ) {
        node.getRgb().apply( this );
    }

    addLine( " );");
    newline();
    }

    public void caseASetbgcolourStatement( ASetbgcolourStatement node ) {

    addLine( "gw.setBGColour( " );

    if ( node.getRgb() != null ) {
        node.getRgb().apply( this );
    }

    addLine( " );" );
    newline();
    }

    public void caseARgb ( ARgb node ) {

    if ( node.getR() != null ) {
        node.getR().apply( this );
    }

    addLine( " , " );

    if ( node.getG() != null ) {
        node.getG().apply( this );
    }

    addLine( " , " );

    if ( node.getB() != null ) {
        node.getB().apply( this );
    }
    }

    /**
     * The first time the translator is run, no code
     * is generated for class and function declarations.
     * Call this to generate them, this allows us to
     * generate code that groups the functions and classes nicely
     * at the bottom of the file.
     */
    public void generateClassFuncDecs()
    {
        genClassFuncDecs = true;
        for (Enumeration e = functions.elements() ; e.hasMoreElements() ;) {
            addLine("\n");
            Node n = (Node)e.nextElement();
            n.apply(this);
        }

        addLine("\n}\n");

        for (Enumeration e = classes.elements() ; e.hasMoreElements() ;) {
            Node n = (Node)e.nextElement();
            n.apply(this);
            addLine("\n");
        }
    }

    public void doConsts()
    {
        genConsts = true;
    }

    public Vector getClasses()
    {
        return classes;
    }

    public Vector getFunctions()
    {
        return functions;
    }

    public boolean usingGraphics()
    {
        return usingGraphics;
    }
}
