/*********************************************************************************
 *   Kenya - an introductory programming language and development environment    *
 *   Copyright (C) 2001 Robert Chatley                                           *
 *                                                                               *
 *   This program is free software; you can redistribute it and/or               *
 *   modify it under the terms of the GNU General Public License                 *
 *   as published by the Free Software Foundation; either version 2              *
 *   of the License, or (at your option) any later version.                      *
 *                                                                               *
 *   This program is distributed in the hope that it will be useful,             *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *   GNU General Public License for more details.                                *
 *                                                                               *
 *   You should have received a copy of the GNU General Public License           *
 *   along with this program; if not, write to the Free Software                 *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
 *                                                                               *
 *   The author can be contacted by email at robert@chatley.com                  *
 *                                                                               *
 *********************************************************************************/

package com.chatley.kenya.gui;

import java.io.*;
import java.awt.*;
import java.util.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.filechooser.*;

class CompileListener implements ActionListener
{

    private KenyaUI ui;
    private String lastCompiledKCode;
    private String slash, pathsep, temp, java , kenya, gj, compiler;

    CompileListener(KenyaUI f)
    {
        ui = f;
        lastCompiledKCode = "";
        slash = System.getProperty("file.separator");
        pathsep = System.getProperty("path.separator");
        temp = ui.prop.getProperty("working.dir");
        java = ui.prop.getProperty("java.location");
        kenya = ui.prop.getProperty("kenya.location");
        gj = ui.prop.getProperty("gj.location");
        compiler = ui.prop.getProperty("compiler");
    }

    public void actionPerformed( ActionEvent event )
    {
        if ( ! ( ui.kenyaArea.getText().trim().equals( ui.lastCompiledKCode ) ) ) {
            if(!ParseListener.parse()) return; // return if parse errors
        }

        if(  ui.javaArea.getText().trim().equals("") ) {
            ui.msgsArea.setText("No code to compile.");
            return;
        }

        ui.setCursor( new Cursor( Cursor.WAIT_CURSOR ) );

        //create package directory

        // cla97 05/10/2001 >>
        /*
        File dir = new File(temp + slash + ui.packagename);
        if ( ! dir.exists() ) {
            dir.mkdir();
        }
        */
        // cla97 05/10/2001 <<

        //write java code to file
        try
        {
            File file = new File(temp + slash  /*ui.packagename + slash*/ + ui.packagename + ".java");

            if ( ! file.exists() ) {
                file.createNewFile();
            }

            FileWriter outfile = new FileWriter( file );
            outfile.write( ui.javaArea.getText() );
            outfile.close();
        }
        catch (IOException e){
            JOptionPane.showMessageDialog( ui , "Error" , "An error occurred writing a file." , JOptionPane.ERROR_MESSAGE );
            return;
        }

        //compile that class

        Process comp = null;

        try {

            if ( compiler.equals("gj") )
            {
                String command = java + " -cp " + gj + slash + "classes" + pathsep + kenya + " gjc.Main -d " +
                             temp + " " + temp + slash  /*ui.packagename + slash*/ + ui.packagename + ".java";
		System.err.println("About to exec:" + command);

                comp = Runtime.getRuntime().exec(command);
                if ( comp == null ) {
                    System.err.println("Unable to spawn external process. Error executing command: " + command );
                    return;
                }

                comp.waitFor();

            } else if ( compiler.equals("javac") ) {

                String javac = ui.prop.getProperty("javac.location");
                String command = javac + " -classpath " + kenya + " -d " + temp + " " + temp + slash /*ui.packagename + slash*/ + ui.packagename + ".java";
                comp = Runtime.getRuntime().exec(command);

                if ( comp == null ) {
                    System.err.println("Could not spawn external process, error executing command: " + command );
                }
                comp.waitFor();
                // Changes by cla97 4/10/2001 >>

                // Becuase the -d option has been used to compile we
                // will get copies of the kenya libraries in the compilation
                // path. We now remove them.

                File graphicsFile = new File(temp + slash + "kenya" + slash + "io" + slash + "GraphicsWindow.class");
                graphicsFile.delete();

                File inputReaderFile = new File(temp + slash + "kenya" + slash + "io" + slash + "InputReader.class");
                inputReaderFile.delete();

                File ioDir = new File(temp + slash + "kenya" + slash + "io" + slash);
                ioDir.delete();

                File kenyaDir = new File(temp + slash + "kenya" + slash);
                kenyaDir.delete();
                // Changes by cla97 4/10/2001 <<


            }
            else {
                System.err.println( "Unknown compiler:" + compiler );
                return;
            }
        }
        catch(Exception e)
        {
            ui.msgsArea.setText("Error on compiling");
            return;
        }

        String line;
        ui.msgsArea.setText("");

        try
        {

            BufferedReader inStream = new BufferedReader( new InputStreamReader( comp.getErrorStream() ));

            while (( line = inStream.readLine()) != null ) {
                ui.msgsArea.append(line + "\n");
            }

            inStream.close();
        }

        catch(IOException e)
        {
            System.err.println("Error on inStream.readLine()");
            e.printStackTrace();
        }

        if ( comp.exitValue() == 0 ) {
            ui.msgsArea.setText("Compiled OK.");

            ui.lastCompiledKCode = ui.lastParsedKCode;
        }
        ui.setCursor( new Cursor( Cursor.DEFAULT_CURSOR ) );
    }
}

