/* *******************************************************************************
 *   Kenya - an introductory programming language and development environment    *
 *   Copyright (C) 2001,2002 Robert Chatley, Alan Chan                           *
 *                                                                               *
 *   This program is free software; you can redistribute it and/or               *
 *   modify it under the terms of the GNU General Public License                 *
 *   as published by the Free Software Foundation; either version 2              *
 *   of the License, or (at your option) any later version.                      *
 *                                                                               *
 *   This program is distributed in the hope that it will be useful,             *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *   GNU General Public License for more details.                                *
 *                                                                               *
 *   You should have received a copy of the GNU General Public License           *
 *   along with this program; if not, write to the Free Software                 *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
 *                                                                               *
 *   The author can be contacted by email at robert@chatley.com                  *
 *                                                                               *
 *********************************************************************************/

package com.chatley.kenya.gui;

import java.io.*;
import javax.swing.*;

/**
 * writer object which redirects all character data written to it to a
 * Java Swing JTextArea object.&nbsp;Mainly used to add a layer of abstraction
 * between the Kenya interpreter and the destination of its output.
 */

public class JTextAreaWriter extends Writer
{
    private JTextArea mDest;

    /**
     * Creates a new <code>JTextAreaWriter</code> object, which will write
     * output to the specified <code>JTextArea</code> object.
     *
     * @param dest the target JTextArea object.
     */
    public JTextAreaWriter(JTextArea dest)
    {
        mDest = dest;
    }

    /**
     * Flushes the stream.&nbsp;Empty implementation in this class.
     */
    public void flush(){}

    /**
     * Closes and flushes the stream.&nbsp;Empty implementation in this class.
     */
    public void close(){}

    /**
     * Write a portion of an array of characters to the underlying JTextArea
     * object.
     *
     * @param cbuf Array of characters
     * @param off Offset from which to start writing characters
     * @param len Number of characters to write
     */
    public void write(char b[], int off, int len)
    {
        mDest.append(new String(b,off,len));
    }
}