/*
 * KUIErrorMsgWriter.java
 */

package com.chatley.kenya.gui;

/**
 * Kenya User Interface Error Message Writer.&nbsp;Writes error messages to
 * the messages error of the kenya IDE, and highlights the line in orange if
 * the line on which the number on which the error occurred is give too.&nbsp;
 * Used to decouple the Typechecker and data flow analyser from the Kenya IDE.
 *
 * @see ErrorMsgWriter
 * @see StderrErrorMsgWriter
 * @see KenyaUI
 * @see CommandLineProcessor
 * @see ParseListener
 *
 * @author Alan Chan
 */

import javax.swing.*;
import java.awt.*;
import com.chatley.kenya.*;

public class KUIErrorMsgWriter implements ErrorMsgWriter
{
    private KenyaUI mUI = null;

    public KUIErrorMsgWriter(KenyaUI ui)
    {
        mUI = ui;
    }

    /**
     * Prints the error message given to the messages area of the Kenya IDE, and
     * also highlights the line on which the error occurred in orange.
     *
     * @param msg the error message to print.
     * @param line the line in the kenya program on which the error occurred.
     */
    public void giveError(String msg, int line)
    {
        mUI.appendMsg(msg);
        mUI.addLineHighlight(line,Color.orange);
        mUI.scrollTo(line,0);
    }

    /**
     * Prints the error message given to the messages area of the Kenya IDE.
     *
     * @param msg the error message to print.
     */
    public void giveError(String msg)
    {
        mUI.appendMsg(msg);
    }
}