/*********************************************************************************
 *   Kenya - an introductory programming language and development environment    *
 *   Copyright (C) 2001 Robert Chatley                                           *
 *                                                                               *
 *   This program is free software; you can redistribute it and/or               *
 *   modify it under the terms of the GNU General Public License                 *
 *   as published by the Free Software Foundation; either version 2              *
 *   of the License, or (at your option) any later version.                      *
 *                                                                               *
 *   This program is distributed in the hope that it will be useful,             *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *   GNU General Public License for more details.                                *
 *                                                                               *
 *   You should have received a copy of the GNU General Public License           *
 *   along with this program; if not, write to the Free Software                 *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
 *                                                                               *
 *   The author can be contacted by email at robert@chatley.com                  *
 *                                                                               *
 *********************************************************************************/

// >>> askc98 2002/2
package com.chatley.kenya.gui;
// <<< askc98

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Properties;
import com.chatley.kenya.gui.syntaxHighlight.*;
import com.chatley.kenya.interpreter.value.Value;
import java.io.File;

/*** debug, remove later*/
import java.io.PushbackReader;
import java.io.StringReader;

import javax.swing.*;

import com.chatley.kenya.minijava.node.*;
import com.chatley.kenya.minijava.lexer.*;
import com.chatley.kenya.minijava.parser.*;
import com.chatley.kenya.interpreter.*;
import com.chatley.kenya.*;
import java.io.*;

// >>>> end


public class KenyaUI extends JFrame implements ExecutionListener
{
    JSplitPane    pane;
    JTabbedPane   tabbedPane;
    KenyaTextArea kenyaArea;
    KenyaTextArea javaArea;
    JTextArea     msgsArea;

    // panels to hold the status bars + text areas
    JPanel        mKenyaPanel;
    JPanel        mJavaPanel;

    // status bars
    StatusBar     mKenyaStatusBar;
    StatusBar     mJavaStatusBar;

    JScrollPane msgsScrollPane;

    // the interpreter output window
    KOutputWindow outwin = null;

    JToolBar toolbar;
    OutputWindow out;
    Properties prop;

    String lastParsedKCode , lastCompiledKCode , packagename;
    int fontheight;

    private File currentDirectory;

    public KenyaUI( Properties p )
    {
        super("Kenya");

        setIconImage(Util.getKenyaIcon().getImage());

        prop = p;
        out = new OutputWindow(this);
        out.outputArea.setEditable(false);

        packagename = "";
        lastParsedKCode = "";
        lastCompiledKCode = "";

        Util.advanceSplashProgress();

        tabbedPane = new JTabbedPane();
        kenyaArea = new KenyaTextArea();
        javaArea = new KenyaTextArea(KenyaTextArea.JAVA_MODE);

	currentDirectory = new File( prop.getProperty("kenya.base.dir"));

        DocumentListener listen = new DocumentListener()
        {
            public void insertUpdate(DocumentEvent e) {
                SaveListener.dirty=true;
            }
            public void removeUpdate(DocumentEvent e) {
                SaveListener.dirty=true;
            }
            public void changedUpdate(DocumentEvent e) {
                SaveListener.dirty=true;
            }
        };

        kenyaArea.getDocument().addDocumentListener(listen);
        javaArea.getDocument().addDocumentListener(listen);

        kenyaArea.setTabSize(2);
        javaArea.setTabSize(2);

        msgsArea = new JTextArea();
        msgsArea.setEditable(false);

        Util.advanceSplashProgress();

        this.getContentPane().setLayout( new BorderLayout() );

        Font courier = new Font( "Courier", Font.PLAIN, Integer.parseInt ( prop.getProperty( "font.size" , "14") ) );

        fontheight = kenyaArea.getFontMetrics( courier ).getHeight();

        kenyaArea.setFont( courier );
        javaArea.setFont( courier );
        msgsScrollPane = new JScrollPane(msgsArea);

        int uiHeight =  Integer.parseInt( prop.getProperty( "ui.height" , "590" ) );
        int uiWidth =  Integer.parseInt( prop.getProperty( "ui.width" , "500" ) );

        Util.advanceSplashProgress();

        kenyaArea.setMinimumSize(new Dimension(0,0));

           kenyaArea.setBorder(
                BorderFactory.createCompoundBorder(
                    BorderFactory.createCompoundBorder(
                      BorderFactory.createTitledBorder("Kenya Code"),
                      BorderFactory.createEmptyBorder(5,5,5,5)),
                      kenyaArea.getBorder()));

            javaArea.setBorder(
                BorderFactory.createCompoundBorder(
                    BorderFactory.createCompoundBorder(
                      BorderFactory.createTitledBorder("Java Code"),
                      BorderFactory.createEmptyBorder(5,5,5,5)),
                      javaArea.getBorder()));

        mKenyaPanel = new JPanel();
        mKenyaPanel.setLayout(new BorderLayout());
        kenyaArea.addCaretListener(mKenyaStatusBar = new StatusBar());
        mKenyaPanel.add(kenyaArea,BorderLayout.CENTER);
        mKenyaPanel.add(mKenyaStatusBar,BorderLayout.SOUTH);

        Util.advanceSplashProgress();

        mJavaPanel = new JPanel();
        mJavaPanel.setLayout(new BorderLayout());
        javaArea.addCaretListener(mJavaStatusBar = new StatusBar());
        mJavaPanel.add(javaArea,BorderLayout.CENTER);
        mJavaPanel.add(mJavaStatusBar,BorderLayout.SOUTH);

        toolbar = new KenyaToolBar(this);
        tabbedPane.addTab(" Kenya ", null , mKenyaPanel, "Write your Kenya code here");
        tabbedPane.addTab(" Java  ", null, mJavaPanel, "See Java code here");

        tabbedPane.setPreferredSize(new Dimension( uiWidth , uiHeight - 150 ));
        tabbedPane.setMinimumSize(new Dimension(0,250));

        msgsArea.setBorder( BorderFactory.createEtchedBorder( javax.swing.border.EtchedBorder.LOWERED ) );

        msgsScrollPane.setPreferredSize( new Dimension( uiWidth , 100 ) );
        msgsScrollPane.setVerticalScrollBarPolicy(
                            JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
        msgsScrollPane.setBorder(
                BorderFactory.createCompoundBorder(
                    BorderFactory.createCompoundBorder(
                                    BorderFactory.createTitledBorder("Messages"),
                                    BorderFactory.createEmptyBorder(3,3,3,3)),
                    msgsArea.getBorder()));

        pane = new JSplitPane( JSplitPane.VERTICAL_SPLIT ,tabbedPane, msgsScrollPane );

        this.getContentPane().add(pane, BorderLayout.CENTER);
        this.getContentPane().add(toolbar, BorderLayout.NORTH);

        Util.advanceSplashProgress();

        // the commented code below adds a floating window with a button which
        // prints the AST of a kenya program to stdout when clicked. Purely
        // for development purposes.

        //JFrame debug = new JFrame("Debug");
        /////////////////////////////////////////////////////////////////////
        //JButton k = new JButton("Tree print");
        //k.addActionListener(new ActionListener(){
        /*    public void actionPerformed(ActionEvent e1){

                try
                {
                    Lexer lexer = new Lexer( new PushbackReader( new StringReader(
                           Util.trimCarriageReturns(kenyaArea.getText()) )));
                    Parser parser = new Parser(lexer);
                    Start ast = parser.parse();

                    PrintWalker a = new PrintWalker();
                    ast.apply(a);

                }
                catch (ParserException e )
                {
                    msgsArea.append( e.getMessage() );

                    //try
                    //{
                        //java.awt.Point start = new java.awt.Point( 0 , e.getToken().getLine() * fontheight );
                        //java.awt.Point end = new java.awt.Point( kenyaArea.getWidth() , e.getToken().getLine() * fontheight  );
                        //highlight.addHighlight( kenyaArea.viewToModel( start ) , kenyaArea.viewToModel( end ) , painter );
                        kenyaArea.addLineHighlight(e.getToken().getLine(),Color.orange);
                        kenyaArea.scrollTo(e.getToken().getLine(),0);
                        setCursor( new Cursor( Cursor.DEFAULT_CURSOR ) );
                    //}
                    //catch ( javax.swing.text.BadLocationException ble ) { System.err.println(ble); }
                }
                catch (LexerException e )
                {
                    msgsArea.append( e.getMessage() );

                    //try
                    //{
                        //java.awt.Point start = new java.awt.Point( 0 , e.line * fontheight );
                        //java.awt.Point end = new java.awt.Point( kenyaArea.getWidth() , e.line * fontheight  );
                        //highlight.addHighlight( kenyaArea.viewToModel( start ) , kenyaArea.viewToModel( end ) , painter );
                        kenyaArea.addLineHighlight(e.line,Color.orange);
                        kenyaArea.scrollTo(e.line,0);
                        setCursor( new Cursor( Cursor.DEFAULT_CURSOR ) );

                    //} catch ( javax.swing.text.BadLocationException ble ) { System.err.println(ble); }

                }
                catch ( java.io.IOException e ) {
                    msgsArea.append("Error parsing program. Probable syntax error.");
                }
                catch ( Exception e ) {
                    System.err.println( e );
                    e.printStackTrace();
                }
            }
        });

        debug.getContentPane().setLayout(new BorderLayout());
        debug.getContentPane().add(j,BorderLayout.NORTH);
        debug.getContentPane().add(k,BorderLayout.SOUTH);
        debug.pack();
        debug.show();*/
    }

    /**
     * overrides show() in JComponent, forces the kenya text area to
     * have focus on startup.
     */
    public void show()
    {
        super.show();
        kenyaArea.requestFocus();
    }

    /**
     * Sets the text in the messages area.
     * @param s the text to set.
     */
    public void setMsg(String s)
    {
        msgsArea.setText(s);
    }

    /**
     * Appends the supplied text to the messages area.
     * @param s the text to append.
     */
    public void appendMsg(String s)
    {
        msgsArea.append(s);
    }

    /**
     * Add user defined highlighting to kenya area on specified line in the
     * specified colour.
     * @param line the line to remove highlighting from
     * @param c the colour to highlight in.
     */
    public void addLineHighlight(int line,Color c)
    {
        kenyaArea.addLineHighlight(line,c);
    }

    /**
     * Removes user defined highlighting from kenya area on specified line.
     * @param line the line to remove highlighting from
     */
    public void removeLineHighlight(int line)
    {
        kenyaArea.removeLineHighlight(line);
    }

    /**
     * Removes all user defined highlighting from kenya area.
     */
    public void removeAllLineHighlights()
    {
        kenyaArea.removeAllLineHighlights();
    }

    /**
     * Scrolls the kenya text area so that the specified line and offset is
     * visible on the screen, so long as the line and offset are valid.
     * @param line the line to scroll to
     * @param offset the offset on the specified line to scroll to
     */
    public void scrollTo(int line, int offset)
    {
        kenyaArea.scrollTo(line,offset);
    }

    public KenyaTextArea kenyaArea()
    {
        return kenyaArea;
    }

    public KenyaTextArea javaArea()
    {
        return javaArea;
    }

    public void setProperites(Properties prop)
    {
        this.prop = prop;
    }

    public Properties getProperties()
    {
        return prop;
    }

    public File getCurrentDirectory() {
	return currentDirectory;
    }

    public void setCurrentDirectory( File p_dir ) {
	currentDirectory = p_dir;
    }

    public void setPackageName(String name)
    {
        packagename = name;
    }

    public String getPackageName()
    {
        return packagename;
    }

    //////////// implementation of ExecutionListener interface /////////////////
    // receives interpreter execution events from an ExecutionManager

    // called by the Execution manager when a line of code is about to be
    // executed. Said line is provided so that it can be highlighted in the
    // editor. Note that this method will only be called if the interpreter
    // is operating in step by step mode, and not during normal interpretation
    // mode.
    public void startingStep(int line){
        addLineHighlight(line,Color.cyan);
        try{
            scrollTo(line,0);
        }catch(Exception e){e.printStackTrace();}
    }

    // similarly, this method is called when the interpreter has finished
    // execution of a particular line of code, so that we can unhighlight it
    // in the editor as appropriate.
    public void endingStep(int line){
        removeLineHighlight(line);
    }

    public void variableChanged(String varName, String scope, Value v){}

    public void interpreterFinished(){}

    public void breakPointSet(int line)
    {
        kenyaArea.setBreakpoint(line);
    }

    public void breakPointRemoved(int line)
    {
        kenyaArea.removeBreakpoint(line);
    }

    public void clearBreakpoints()
    {
        kenyaArea.clearBreakpoints();
    }

    public void functionEntered(String scope){}

    public void functionExited(String scope){}

    // <<< end /////////////////////////////////////////////////////////////////

    /*
     * Mouse handler attached to the message area.
     * When a use double clicks an error message, highlights
     * the line of the error.
     */
    /*private class KMouseHandler extends MouseAdapter
    {
        public void mouseClicked(MouseEvent e)
        {
            if(e.getClickCount()==2)
            {

                Point p = e.getPoint();
                int offset = msgsArea.viewToModel(p);
                System.err.println("offset clicked = " + offset);
                try
                {
                    int line = msgsArea.getLineOfOffset(offset);
                    System.err.println("line clicked = " + line);

                    int start = msgsArea.getLineStartOffset(line);
                    int end   = msgsArea.getLineEndOffset(line);
                    String text = msgsArea.getText(start,end-start);
                    System.err.println("Text = " + text);
                }
                catch(BadLocationException be){be.printStackTrace();}
            }
        }
    }*/
}
