/*********************************************************************************
 *   Kenya - an introductory programming language and development environment    *
 *   Copyright (C) 2001 Robert Chatley                                           *
 *                                                                               *
 *   This program is free software; you can redistribute it and/or               *
 *   modify it under the terms of the GNU General Public License                 *
 *   as published by the Free Software Foundation; either version 2              *
 *   of the License, or (at your option) any later version.                      *
 *                                                                               *
 *   This program is distributed in the hope that it will be useful,             *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *   GNU General Public License for more details.                                *
 *                                                                               *
 *   You should have received a copy of the GNU General Public License           *
 *   along with this program; if not, write to the Free Software                 *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
 *                                                                               *
 *   The author can be contacted by email at robert@chatley.com                  *
 *                                                                               *
 *********************************************************************************/

package com.chatley.kenya.gui;

import java.io.*;
import java.awt.*;
import java.util.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.filechooser.*;

class OpenListener implements ActionListener {
  private KenyaUI ui;

  OpenListener(KenyaUI f)
  {
    ui = f;
  }

  public void actionPerformed( ActionEvent event )
  {
    try {

      if (SaveListener.dirty) {
        int save = JOptionPane.showConfirmDialog( ui , "Save current Kenya before opening new project?" , "Open File" , JOptionPane.YES_NO_CANCEL_OPTION );
        if ( save == JOptionPane.YES_OPTION ) {
          SaveListener.save(ui,false);
        } else if (save == JOptionPane.CANCEL_OPTION) {
          return;
        }
      }

      if(ui.outwin!=null){
        int destroy = JOptionPane.showConfirmDialog( ui , "Opening new file will terminate running interpreter instance." , "Terminate interpreter" , JOptionPane.YES_NO_CANCEL_OPTION );
        if ( destroy == JOptionPane.YES_OPTION ) {
            ui.outwin.dispose();
            ui.outwin = null;
            ui.removeAllLineHighlights();
        } else if (destroy == JOptionPane.CANCEL_OPTION ||
                   destroy == JOptionPane.NO_OPTION) {
          return;
        }
      }

      File basedir = ui.getCurrentDirectory(); //new File( ui.prop.getProperty("kenya.base.dir") );
      JFileChooser fc = new JFileChooser(basedir);
      ExampleFileFilter filter = new ExampleFileFilter();
      filter.addExtension("java");
      filter.setDescription("Java files");
      fc.addChoosableFileFilter(filter);

      filter = new ExampleFileFilter();
      filter.addExtension("k");
      filter.setDescription("Kenya files");
      fc.addChoosableFileFilter(filter);

      int returnVal = fc.showOpenDialog(ui);
     
 
      if (returnVal == JFileChooser.APPROVE_OPTION) {
	  ui.setCurrentDirectory( fc.getCurrentDirectory() );
	  File file = fc.getSelectedFile();
	  SaveListener.file = file;
	  BufferedReader infile = new BufferedReader( new FileReader( file ) );
	  
	  String filename = file.getName();
	  String extension = "";
	  ui.setTitle( "Kenya - " + filename );
	  
	  if ( filename.indexOf(".") >= 0 ) {
	      ui.packagename = filename.substring( 0 , filename.indexOf(".") );
	      extension = filename.substring( filename.indexOf(".") + 1 , filename.length() );
	  } else {
	      ui.packagename = filename;
	  }

        ui.kenyaArea.setText("");
        ui.javaArea.setText("");
        ui.msgsArea.setText("Opened file " + filename);

        // temporarily switch off line counting for the java area
        // (speeds things up a bit =D )
        ui.mKenyaStatusBar.setCountLines(false);
        ui.mJavaStatusBar.setCountLines(false);

        if ( extension.equals("java") ) {
          while ( infile.ready() ) {
            ui.javaArea.append( infile.readLine() );
            ui.javaArea.append("\n");
          }
          ui.tabbedPane.setSelectedIndex( 1 );
          ui.javaArea.setCaretPosition( 0 );

        } else {
          while ( infile.ready() ) {
            ui.kenyaArea.append( infile.readLine() );
            ui.kenyaArea.append("\n");
          }
          ui.tabbedPane.setSelectedIndex( 0 );
          ui.kenyaArea.setCaretPosition( 0 );
        }

        infile.close();
        // So that when the user has loaded but NOT changed
        // a file they will not be asked to save it.
        SaveListener.dirty = false;
      }
    }

    catch ( FileNotFoundException fnf ) {
      JOptionPane.showMessageDialog(ui, "File not found." , "Error" , JOptionPane.ERROR_MESSAGE );
    } catch ( IOException e ) {
      JOptionPane.showMessageDialog(ui, "An error occurred while reading the file." , "Error" , JOptionPane.ERROR_MESSAGE );
    }
    finally{
        ui.mKenyaStatusBar.setCountLines(true);
        ui.mJavaStatusBar.setCountLines(true);
        ui.removeAllLineHighlights();
        ui.clearBreakpoints();
    }
  }
}
