/*********************************************************************************
 *   Kenya - an introductory programming language and development environment    *
 *   Copyright (C) 2001 Robert Chatley                                           *
 *                                                                               *
 *   This program is free software; you can redistribute it and/or               *
 *   modify it under the terms of the GNU General Public License                 *
 *   as published by the Free Software Foundation; either version 2              *
 *   of the License, or (at your option) any later version.                      *
 *                                                                               *
 *   This program is distributed in the hope that it will be useful,             *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *   GNU General Public License for more details.                                *
 *                                                                               *
 *   You should have received a copy of the GNU General Public License           *
 *   along with this program; if not, write to the Free Software                 *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
 *                                                                               *
 *   The author can be contacted by email at robert@chatley.com                  *
 *                                                                               *
 *********************************************************************************/

package com.chatley.kenya.gui;

import javax.swing.*;
import javax.swing.text.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;

class OutputWindow extends JFrame implements KenyaOutputWindow
{

    JTextArea outputArea;
    JScrollPane outputScrollPane;
    InputBox inputbox;

    private KenyaUI ui;
    Process proc;
    BufferedWriter outStream;
    private PrintWriter mPrintWriter = null;

    OutputWindow( KenyaUI u )
    {

        super("Kenya Output");
        setIconImage(Util.getKenyaIcon().getImage());
        ui = u;
        proc = null;
        outStream = null;

        getContentPane().setLayout( new BorderLayout() );

        outputArea = new JTextArea();
        inputbox = new InputBox( this );

        Font courier = new Font( "Courier", Font.PLAIN, Integer.parseInt ( ui.prop.getProperty( "font.size" , "14") ) );

        inputbox.setFont( courier );

        outputArea.setMargin( new Insets( 5, 5, 5, 5 ) );
        outputArea.setFont( courier );
        outputArea.setLineWrap(true);
        outputArea.setWrapStyleWord(true);

        outputScrollPane = new JScrollPane(outputArea);
        int sizex = Integer.parseInt(ui.prop.getProperty("outputwindow.width","500"));
        int sizey = Integer.parseInt(ui.prop.getProperty("outputwindow.height","300"));

        outputScrollPane.setPreferredSize( new Dimension ( sizex, sizey ) );
        outputScrollPane.setVerticalScrollBarPolicy(
          JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        outputScrollPane.setBorder(
          BorderFactory.createCompoundBorder(
            BorderFactory.createCompoundBorder(
              BorderFactory.createTitledBorder("Program Output"),
              BorderFactory.createEmptyBorder(5,5,5,5)),
              outputScrollPane.getBorder()));

        getContentPane().add( outputScrollPane , BorderLayout.CENTER );

        JPanel inputArea = new JPanel(new BorderLayout());
        inputArea.add(inputbox,BorderLayout.CENTER);
        inputArea.add(new EOFButton(this),BorderLayout.EAST);
        getContentPane().add( inputArea , BorderLayout.SOUTH );

        setLocation(660,220);

        addWindowListener(new WindowAdapter() {
        public void windowActivated(WindowEvent e){
            // query the font size and set it
            outputArea.setFont(ui.kenyaArea().getPainter().getFont());
        }

        public void windowClosing(WindowEvent e) {
            outputArea.setText("");
            ui.msgsArea.setText("");
            proc.destroy();
            setVisible(false);
        }
        });
    }

    void connectProcess( Process p )
    {
        proc = p;
        outStream = new BufferedWriter( new OutputStreamWriter ( proc.getOutputStream() ) );
        if ( outStream == null ) { JOptionPane.showMessageDialog( ui , "Trying to connect to null stream" ); }
    }

    // -------- implementation of the KenyaOutputWindow interface -------
    public PrintWriter getOutputWriter()
    {
        if(mPrintWriter == null) mPrintWriter = new PrintWriter(outStream);
        return mPrintWriter;
    }

    public void appendText(String s)
    {
        outputArea.append(s);
    }
}
