/*********************************************************************************
 *   Kenya - an introductory programming language and development environment    *
 *   Copyright (C) 2001 Robert Chatley                                           *
 *                                                                               *
 *   This program is free software; you can redistribute it and/or               *
 *   modify it under the terms of the GNU General Public License                 *
 *   as published by the Free Software Foundation; either version 2              *
 *   of the License, or (at your option) any later version.                      *
 *                                                                               *
 *   This program is distributed in the hope that it will be useful,             *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *   GNU General Public License for more details.                                *
 *                                                                               *
 *   You should have received a copy of the GNU General Public License           *
 *   along with this program; if not, write to the Free Software                 *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
 *                                                                               *
 *   The author can be contacted by email at robert@chatley.com                  *
 *                                                                               *
 *********************************************************************************/

// >>> askc98 2002/2
package com.chatley.kenya.gui;
// <<< askc98

import java.util.*;
import java.awt.event.*;
import java.awt.Cursor;
import java.io.PushbackReader;
import java.io.StringReader;

import javax.swing.*;
import java.awt.Color;

import com.chatley.kenya.*;
import com.chatley.kenya.minijava.node.*;
import com.chatley.kenya.minijava.lexer.*;
import com.chatley.kenya.minijava.parser.*;

import com.chatley.kenya.interpreter.*;

class ParseListener implements ActionListener
{
    private static KenyaUI ui;

    ParseListener(KenyaUI f)
    {
        ui = f;
    }

    public void actionPerformed( ActionEvent event )
    {
        parse();
    }

    static boolean parse()
    {
        boolean parsedOk = false;


        if(  ui.kenyaArea.getText().trim().equals("") ) {
            ui.msgsArea.setText("No Kenya to parse.");
            return false;
        }
        // cla97 05/10/2001 - Fix for Save As >>
        /*if ( ui.kenyaArea.getText().trim().equals( ui.lastParsedKCode ) ) {
            ui.msgsArea.setText("Kenya unchanged since last parse.");
            return;
        }*/
        // cla97 05/10/2001 <<

        // in other words, if the file hasn't been saved, pop up
        // a dialog box and prompt to save file. Get the filename (sans extension)and
        // store it in ui.packagename and use it later for class name
        // of translated kenya program and stuff...
        while ( ui.packagename.equals("") ) {
            // cla97 05/10/2001 >>
            /*
            String p = JOptionPane.showInputDialog( ui , "Please enter a filename:");
            if ( p == null ) { return; }
            if ( p.equals( "") || ! Util.isValidIdentifier( p ) ) {
            JOptionPane.showMessageDialog( ui , "Invalid filename (only letters are allowed)" );
            continue;
            }
            ui.packagename = p;
            */
            SaveListener.save(ui,true);
            // cla97 05/10/2001 <<
        }

        ui.setCursor( new Cursor( Cursor.WAIT_CURSOR ) );
        ui.lastParsedKCode = ui.kenyaArea.getText().trim();

        try
        {
            ui.msgsArea.setText("");
            ui.javaArea.setText("");

            Lexer lexer = new Lexer( new PushbackReader( new StringReader(
                   Util.trimCarriageReturns(ui.kenyaArea.getText()) ),5000));
            Parser parser = new KParser(lexer);
            Start ast = parser.parse();

            ui.kenyaArea.removeAllLineHighlights();

            // Typecheck

            KUIErrorMsgWriter msgWriter = new KUIErrorMsgWriter(ui);

            KTypeChecker tc = new KTypeChecker(msgWriter,ast);

            // tc.execute() returns a boolean indicating whether the
            // kenya program is type correct.
            if (tc.execute()) {
                // data flow analysis
                DataFlowAnalyser dfa = new DataFlowAnalyser(msgWriter);
                ast.apply(dfa);

                if(!dfa.errorsHaveOccurred())
                {

                   Translator t = new Translator(tc.getTypeChecker());

                   //Translate

                   ast.apply(t);

                   t.generateClassFuncDecs();

                   // temporarily switch off line counting for the java area
                   // (speeds things up a bit =D )
                   ui.mJavaStatusBar.setCountLines(false);

                   Vector v = t.getCode();
                   Iterator i = t.getImports();

                   //cla97 05/10/2001 >>
                   //ui.javaArea.setText("package " + ui.packagename + ";\n\n");
                   //cla97 05/10/2001 <<
                   if (  i.hasNext() )
                   {

                       if ( ! ( ui.prop.getProperty("compiler").equals("gj") ) ) {
                           ui.msgsArea.append("Warning: Your compiler (" + ui.prop.getProperty("compiler") +
                             ") does not support parameterised types.\n");
                       }

                       while ( i.hasNext() ) {
                           ui.javaArea.append( "import gj.util." + ((String)i.next()).trim() + ";\n" );
                       }

                       ui.javaArea.append( "\n" );
                   }

                   if ( t.usingGraphics() )
                       ui.javaArea.append( "import kenya.GraphicsWindow;\n\n");

                   if ( t.getReads() )
                   {
                       ui.javaArea.append( "import kenya.io.InputReader;\n\n" );
                   }
                   //cla97 05/10/2001 >>
                   ui.javaArea.append("public class "+ui.packagename +" {\n");
                   //cla97 05/10/2001 <<
                   Iterator constit  = t.getConstants();

                   while ( constit.hasNext() )
                   {
                       Translator forconsts = new Translator();
                       forconsts.doConsts();
                       ((Node)constit.next()).apply( forconsts );
                       for (Enumeration e = forconsts.getCode().elements() ; e.hasMoreElements() ; ) {
                           CodeLine c = (CodeLine)e.nextElement();
                           ui.javaArea.append( c.code );
                       }
                   }

                   for (Enumeration e = v.elements() ; e.hasMoreElements() ; )
                   {
                       CodeLine c = (CodeLine)e.nextElement();

                       if ( c.newline )
                       {
                           ui.javaArea.append("\n");

                           if ( ! e.hasMoreElements() ) {
                               break;
                           }

                           CodeLine next = (CodeLine)e.nextElement();

                           for ( int j = 0 ; j < next.indent ; j++ )
                               ui.javaArea.append("\t");

                           if ( next.code.equals("graphics") )
                           {
                               if ( t.usingGraphics() )
                               {
                                   ui.javaArea.append("GraphicsWindow gw = new GraphicsWindow();\n\n");
                                   for ( int j = 0 ; j < next.indent ; j++ )
                                       ui.javaArea.append("\t");
                               }

                           } else {
                               ui.javaArea.append( next.code );
                           }
                       }
                       ui.javaArea.append( c.code );
                   }// end for
                }
            }
            ui.msgsArea.append("Finished parsing.");
            ui.setCursor( new Cursor( Cursor.DEFAULT_CURSOR ) );
            parsedOk = true;
        }
        catch (ParserException e )
        {
            ui.msgsArea.append( e.getMessage() );
            ui.kenyaArea.addLineHighlight(e.getToken().getLine(),Color.orange);
            ui.kenyaArea.scrollTo(e.getToken().getLine(),0);
            ui.setCursor( new Cursor( Cursor.DEFAULT_CURSOR ) );
            parsedOk = false;
        }
        catch (LexerException e )
        {
            ui.msgsArea.append( e.getMessage() );
            ui.kenyaArea.addLineHighlight(e.line,Color.orange);
            ui.kenyaArea.scrollTo(e.line,0);
            ui.setCursor( new Cursor( Cursor.DEFAULT_CURSOR ) );

            parsedOk = false;
        }
        catch ( java.io.IOException e ) {
            ui.msgsArea.append("Error parsing program. Probable syntax error.");
            e.printStackTrace();
            parsedOk = false;
        }
        catch ( Exception e ) {
            System.err.println( e );
            e.printStackTrace();
            parsedOk = false;
        }
        finally
        {
            ui.setCursor( new Cursor( Cursor.DEFAULT_CURSOR ) );
            ui.mJavaStatusBar.setCountLines(true);
        }
        return parsedOk;
    }
}
