/* *******************************************************************************
 *   Kenya - an introductory programming language and development environment    *
 *   Copyright (C) 2001,2002 Robert Chatley, Alan Chan                           *
 *                                                                               *
 *   This program is free software; you can redistribute it and/or               *
 *   modify it under the terms of the GNU General Public License                 *
 *   as published by the Free Software Foundation; either version 2              *
 *   of the License, or (at your option) any later version.                      *
 *                                                                               *
 *   This program is distributed in the hope that it will be useful,             *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *   GNU General Public License for more details.                                *
 *                                                                               *
 *   You should have received a copy of the GNU General Public License           *
 *   along with this program; if not, write to the Free Software                 *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
 *                                                                               *
 *   The author can be contacted by email at robert@chatley.com                  *
 *                                                                               *
 *********************************************************************************/

/*
 * SplashScreen.java - Splash screen
 * Alan Chan 2002/6
 */

package com.chatley.kenya.gui;

import javax.swing.border.*;
import javax.swing.*;
import java.awt.*;
import java.net.URL;
import java.util.Random;

public class SplashScreen extends JWindow
{
	public SplashScreen()
	{
		setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));

		JPanel splash = new JPanel(new BorderLayout());
		URL url = getClass().getResource("/images/kenya_logo.gif");
		if(url != null)
		{
			splash.add(new JLabel(new ImageIcon(url)),
				BorderLayout.CENTER);
		}

		progress = new JProgressBar(0,7); // 7 steps before full
		progress.setStringPainted(true);
		progress.setBorderPainted(false);
		progress.setString("Kenya is starting up...");
		progress.setBackground(Color.white);
		splash.add(BorderLayout.SOUTH,progress);

		splash.setBorder(new MatteBorder(1,1,1,1,Color.black));

		setContentPane(splash);

		Dimension screen = getToolkit().getScreenSize();
		pack();
		setLocation((screen.width - getSize().width) / 2,
			(screen.height - getSize().height) / 2);
		show();
	}

	public void advance()
	{
		try
		{
			SwingUtilities.invokeAndWait(new Runnable() {
				public void run()
				{
					progress.setValue(progress.getValue() + 1);
				}
			});
			Thread.yield();
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
	}

	// private members
	private JProgressBar progress;


	public static void main(String[] args)
	{
	   new SplashScreen();
    }
}
