/* *******************************************************************************
 *   Kenya - an introductory programming language and development environment    *
 *   Copyright (C) 2001,2002 Robert Chatley, Alan Chan                           *
 *                                                                               *
 *   This program is free software; you can redistribute it and/or               *
 *   modify it under the terms of the GNU General Public License                 *
 *   as published by the Free Software Foundation; either version 2              *
 *   of the License, or (at your option) any later version.                      *
 *                                                                               *
 *   This program is distributed in the hope that it will be useful,             *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *   GNU General Public License for more details.                                *
 *                                                                               *
 *   You should have received a copy of the GNU General Public License           *
 *   along with this program; if not, write to the Free Software                 *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
 *                                                                               *
 *   The author can be contacted by email at robert@chatley.com                  *
 *                                                                               *
 *********************************************************************************/

package com.chatley.kenya.gui;

import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.text.*;
import com.chatley.kenya.gui.syntaxHighlight.jeditSyntax.*;
/**
 * The status bar
 */

public class StatusBar extends JPanel implements CaretListener
{
    /** The line number label */
    protected JLabel mLine;

    /** The message label */
    protected JLabel mMessage;

    /** The main message */
    protected String mMainMessage;

    /** Whether we want to count lines or not. */
    protected boolean mCountLines = true;

    /**
     * Creates a new status bar
     */
    public StatusBar()
    {
        super(new BorderLayout(0, 0));

        mLine = new JLabel("Ln: 1, Char: 1");
        mLine.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
        mLine.setPreferredSize(new Dimension(100, 22));
        add("West", mLine);

        JPanel p = new JPanel(new BorderLayout(0, 0));
        mMessage = new JLabel();
        mMessage.setBorder(BorderFactory.createBevelBorder(BevelBorder.LOWERED));
        p.add(mMessage);
        add(p);
    }

    /**
     * Sets whether we should display current line number in status bar.
     * @param enabled whether line counting is enabled.
     */
    public void setCountLines(boolean enabled)
    {
       mCountLines = enabled;
       mLine.setEnabled(enabled);
       //if(mCountLines) System.err.println("Line counting is on");
       //else System.err.println("Line counting is off");
    }

    /**
     * Returns whether line counting is enabled
     */
    public boolean getCountLines()
    {
        return mCountLines;
    }

    /**
     * Sets the line number and character number message
     * @param n the line number
     * @param c the char number
     */
    public void setLine(int n, int c) {
        mLine.setText("Ln: " + n + ", Char: " + c);
    }

    /**
     * Sets a temporary message
     * @param s the message
     */
    public void setMessage(String s) {
        mMessage.setText(s);
        new DisplayThread().start();
    }

    /**
     * Sets a temporary message to display
     * @param s  the message
     * @param s2 a string to concatenate with the message
     */
    public void setMessage(String s, String s2) {
        mMessage.setText( s + " " + s2);
        new DisplayThread().start();
    }

    /**
     * Sets the main message
     * @param s the message
     */
    public void setMainMessage(String s) {
        mMainMessage = s;
        mMessage.setText(mMainMessage);
    }

    /**
     * Sets the main message to display
     * @param s  the message
     * @param s2 a string to concatenate with the message
     */
    public void setMainMessage(String s, String s2)
    {
        mMainMessage = s + " " + s2;
        mMessage.setText(mMainMessage);
    }

    /** for testing */
    public static void main(String[] args)
    {
        JFrame f = new JFrame();
        f.getContentPane().add(new StatusBar());
        f.show();
    }

    /**
     * Implements the CaretListener interface. Listens
     * for changes in the position of the caret in the
     * text component this status is associated with, and
     * updates the line number displayed accordingly.
     */
    public void caretUpdate(CaretEvent e)
    {
        //don't do anything if line counting is off
        if(!mCountLines) return;

        JEditTextArea source = (JEditTextArea)e.getSource();
        int ln = source.getCaretLine();
        int col = source.getCaretPosition()+1-source.getLineStartOffset(ln);

        //this.setMainMessage(" p = " + p + " length = " + t.length());
        this.setLine( ln + 1, col);
    }

    /**
     * To display the main message
     */
    protected class DisplayThread extends Thread
    {
        public DisplayThread() {
            setPriority(Thread.MIN_PRIORITY);
        }

        public void run()
        {
            try {
               Thread.sleep(3000);
            }
            catch(InterruptedException e) {
            }
            mMessage.setText(mMainMessage);
        }
    }
}
