/*********************************************************************************
 *   Kenya - an introductory programming language and development environment    *
 *   Copyright (C) 2001 Robert Chatley                                           *
 *                                                                               *
 *   This program is free software; you can redistribute it and/or               *
 *   modify it under the terms of the GNU General Public License                 *
 *   as published by the Free Software Foundation; either version 2              *
 *   of the License, or (at your option) any later version.                      *
 *                                                                               *
 *   This program is distributed in the hope that it will be useful,             *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *   GNU General Public License for more details.                                *
 *                                                                               *
 *   You should have received a copy of the GNU General Public License           *
 *   along with this program; if not, write to the Free Software                 *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
 *                                                                               *
 *   The author can be contacted by email at robert@chatley.com                  *
 *                                                                               *
 *********************************************************************************/

/*
 * Utility functions.
 */

// >>> askc98 2002/2
package com.chatley.kenya.gui;
// <<< askc98

import javax.swing.*;
import java.net.*;
import java.util.HashSet;

public class Util
{
    private static ImageIcon mAppIcon = null;
    private static HashSet mJavaKeywords = new HashSet();
    private static SplashScreen mSplash = null;

    // because the Java API doesn't contain a function which determines
    // whether a given string is a Java reserved word, we implement our
    // own hardcoded lookup table. Note that this shall need to be updated
    // if and when new Java keywords are added to the Java spec.
    static{
        mJavaKeywords.add("abstract"); mJavaKeywords.add("default");
        mJavaKeywords.add("if");       mJavaKeywords.add("private");
        mJavaKeywords.add("this");     mJavaKeywords.add("boolean");
        mJavaKeywords.add("do");       mJavaKeywords.add("implements");
        mJavaKeywords.add("protected");mJavaKeywords.add("throw");
        mJavaKeywords.add("break");    mJavaKeywords.add("double");
        mJavaKeywords.add("import");   mJavaKeywords.add("public");
        mJavaKeywords.add("throws");   mJavaKeywords.add("byte");
        mJavaKeywords.add("else");     mJavaKeywords.add("instanceof");
        mJavaKeywords.add("return");   mJavaKeywords.add("transient");
        mJavaKeywords.add("case");     mJavaKeywords.add("extends");
        mJavaKeywords.add("int");      mJavaKeywords.add("short");
        mJavaKeywords.add("try");      mJavaKeywords.add("catch");
        mJavaKeywords.add("final");    mJavaKeywords.add("interface");
        mJavaKeywords.add("static");   mJavaKeywords.add("void");
        mJavaKeywords.add("char");     mJavaKeywords.add("finally");
        mJavaKeywords.add("long");     mJavaKeywords.add("strictfp");
        mJavaKeywords.add("volatile"); mJavaKeywords.add("class");
        mJavaKeywords.add("float");    mJavaKeywords.add("native");
        mJavaKeywords.add("super");    mJavaKeywords.add("while");
        mJavaKeywords.add("const");    mJavaKeywords.add("for");
        mJavaKeywords.add("new");      mJavaKeywords.add("switch");
        mJavaKeywords.add("continue"); mJavaKeywords.add("goto");
        mJavaKeywords.add("package");  mJavaKeywords.add("synchronized");
        mJavaKeywords.add("null");     mJavaKeywords.add("true");
        mJavaKeywords.add("false");
    }

    private Util(){}


    public static boolean isValidIdentifier( String s ){

        char[] chars = s.toCharArray();

        if ( ! Character.isJavaIdentifierStart( chars[0] ) )
            return false;

        for ( int i = 1 ; i < chars.length ; i++ )
            if ( ! Character.isJavaIdentifierPart( chars[i] ) && chars[i] != '.' )
            return false;

        int theDot = s.indexOf('.');
        String namePart = (theDot!=-1) ? s.substring(0,theDot) : s;
        if(isJavaKeyword(namePart)) return false;

        return true;
    }

    /**
     * Returns a copy of a string will all the '\r' characters removed. If
     * the string does not contain '\r' chars, then the string is returned
     * unchanged.
     * @param s the string to remove '\r' chars from.
     */
    public static String trimCarriageReturns(String s)
    {
        String t = s;
        // remove all stinky '\r' chars

        StringBuffer sb = new StringBuffer();
        for(int i=0;i<t.length();i++)
            if(t.charAt(i)!= '\r') sb.append(t.charAt(i));
        t = sb.toString();

        return t;
    }

    public static ImageIcon getKenyaIcon()
    {
        if(mAppIcon==null){
            URL url = ClassLoader.getSystemResource("images/icon.gif");
            if(url!=null)
                mAppIcon = (new ImageIcon(url));
        }
        return mAppIcon;
    }

    /**
     * returns true iff the given string is a Java reserved word.
     * @param s the string to test.
     */
    public static boolean isJavaKeyword(String s)
    {
        return mJavaKeywords.contains(s);
    }

    /**
     * Displays the kenya splash screen
     */
    public static void showSplash()
    {
        if(mSplash ==null) mSplash = new SplashScreen();
        mSplash.show();
    }

    /**
     * Hides the kenya splash screen
     */
    public static void hideSplash()
    {
        if(mSplash!=null){
            mSplash.dispose();
            mSplash = null;
        }
    }

    /**
     * advances the splash screen progress bar by one unit
     */
    public static void advanceSplashProgress()
    {
        if(mSplash != null)
            mSplash.advance();
    }
}
