/* *******************************************************************************
 *   Kenya - an introductory programming language and development environment    *
 *   Copyright (C) 2001,2002 Robert Chatley, Alan Chan                           *
 *                                                                               *
 *   This program is free software; you can redistribute it and/or               *
 *   modify it under the terms of the GNU General Public License                 *
 *   as published by the Free Software Foundation; either version 2              *
 *   of the License, or (at your option) any later version.                      *
 *                                                                               *
 *   This program is distributed in the hope that it will be useful,             *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *   GNU General Public License for more details.                                *
 *                                                                               *
 *   You should have received a copy of the GNU General Public License           *
 *   along with this program; if not, write to the Free Software                 *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
 *                                                                               *
 *   The author can be contacted by email at robert@chatley.com                  *
 *                                                                               *
 *********************************************************************************/

/*
 * KenyaDefaults.java - Encapsulates default values for kenya
 * See the class TextAreaDefaults in the jeditSyntax package for more details
 * Alan Chan 2002/1
 *
 */

package com.chatley.kenya.gui.syntaxHighlight; // askc98

import com.chatley.kenya.gui.syntaxHighlight.jeditSyntax.*;
import javax.swing.JPopupMenu;
import java.awt.Color;

/**
 * Encapsulates default settings for a text area. This can be passed
 * to the constructor once the necessary fields have been filled out.
 * The advantage of doing this over calling lots of set() methods after
 * creating the text area is that this method is faster.
 */
public class KenyaDefaults extends TextAreaDefaults
{
    /**
     * Returns a new KenyaDefaults object with the default values filled
     * in.
     */
    public KenyaDefaults()
    {
        inputHandler = new DefaultInputHandler();
        inputHandler.addDefaultKeyBindings();
        document = new SyntaxDocument();
        editable = true;

        caretVisible = true;
        caretBlinks = true;
        electricScroll = 3;

        cols = 80;
        rows = 15;
        styles = getKenyaSyntaxStyles();
        caretColor = Color.red;
        selectionColor = new Color(0xccccff);
        lineHighlightColor = new Color(0xe0e0e0);
        lineHighlight = false;//true;
        bracketHighlightColor = new Color(0xD4A0F8); //Color.red;
        bracketHighlight = true;
        eolMarkerColor = new Color(0x009999);
        eolMarkers = false;
        paintInvalid = false;
    }

    /**
     * Returns the default style table. This can be passed to the
     * <code>setStyles()</code> method of <code>SyntaxDocument</code>
     * to use the default syntax styles.
     */
    public static SyntaxStyle[] getKenyaSyntaxStyles()
    {
        SyntaxStyle[] styles = new SyntaxStyle[Token.ID_COUNT];

        styles[Token.COMMENT1] = new SyntaxStyle(Color.green.darker(),false,false);
        styles[Token.COMMENT2] = new SyntaxStyle(Color.green.darker(),false,false);
        styles[Token.KEYWORD1] = new SyntaxStyle(Color.blue,false,false);
        styles[Token.KEYWORD2] = new SyntaxStyle(Color.blue,false,false);
        styles[Token.KEYWORD3] = new SyntaxStyle(Color.blue,false,false);
        styles[Token.LITERAL1] = new SyntaxStyle(new Color(0xB03060),false,false);
        styles[Token.LITERAL2] = new SyntaxStyle(/*new Color(0xB03060)*/Color.magenta,false,false);
        styles[Token.LABEL]    = new SyntaxStyle(new Color(0x990033),false,true);
        styles[Token.OPERATOR] = new SyntaxStyle(Color.black,false,true);
        styles[Token.INVALID]  = new SyntaxStyle(Color.red,false,true);

        return styles;
    }
}
