/**
 * ValueTreeCellRenderer.java
 *
 * A custom TreeCellRenderer for the kenya variable watcher
 * which displays our own icons depending on the variable represented
 * by a node.
 */
package com.chatley.kenya.gui.varview;

import javax.swing.tree.*;
import com.chatley.kenya.interpreter.value.*;
import com.chatley.kenya.gui.Util;
import java.net.*;
import javax.swing.*;
import java.awt.*;

class ValueTreeCellRenderer extends DefaultTreeCellRenderer
  implements ValueSwitch
{
    ImageIcon mClass  = null;
    ImageIcon mArray  = null;
    ImageIcon mString = null;
    ImageIcon mInt    = null;
    ImageIcon mDouble = null;
    ImageIcon mBool   = null;
    ImageIcon mChar   = null;
    ImageIcon mRoot   = null;

    ValueTreeCellRenderer()
    {
        URL url = ClassLoader.getSystemResource("images/array.gif");
        if(url!=null)
            mArray = (new ImageIcon(url));

        url = ClassLoader.getSystemResource("images/class.gif");
        if(url!=null)
            mClass = (new ImageIcon(url));

        url = ClassLoader.getSystemResource("images/integer.gif");
        if(url!=null)
            mInt = (new ImageIcon(url));

        url = ClassLoader.getSystemResource("images/string.gif");
        if(url!=null)
            mString = (new ImageIcon(url));

        url = ClassLoader.getSystemResource("images/char.gif");
        if(url!=null)
            mChar = (new ImageIcon(url));

        url = ClassLoader.getSystemResource("images/double.gif");
        if(url!=null)
            mDouble = (new ImageIcon(url));

        url = ClassLoader.getSystemResource("images/boolean.gif");
        if(url!=null)
            mBool = (new ImageIcon(url));

        mRoot = Util.getKenyaIcon();
    }

    public Component getTreeCellRendererComponent(
                        JTree tree,
                        Object value,
                        boolean sel,
                        boolean expanded,
                        boolean leaf,
                        int row,
                        boolean hasFocus)
    {
        super.getTreeCellRendererComponent(tree,value,sel,expanded,
          leaf,row,hasFocus);

        try{
            DefaultMutableTreeNode node = ((DefaultMutableTreeNode)value);
            if(node.isRoot()){
                if(mRoot!=null) setIcon(mRoot);
            }else{
                Object userObj = node.getUserObject();
                if(userObj instanceof Value) ((Value)userObj).apply(this);
            }
        }catch(Exception e){ e.printStackTrace(); }

        return this;
    }

        // -------- implementation of ValueSwitch interface -----------

    public Object caseAArrayValue(ArrayValue v)
    {
        if(mArray!=null) setIcon(mArray);
        return null;
    }

    public Object caseAClassValue(ClassValue v)
    {
        if(mClass!=null) setIcon(mClass);
        return null;
    }

    public Object caseANullValue(NullValue v)
    {
        if(mClass!=null) setIcon(mClass);
        return null;
    }

    public Object caseAStringValue(StringValue v)
    {
        if(mString!=null) setIcon(mString);
        return null;
    }

    public Object caseAIntValue(IntValue v)
    {
        if(mInt!=null) setIcon(mInt);
        return null;
    }

    public Object caseADoubleValue(DoubleValue v)
    {
        if(mDouble!=null) setIcon(mDouble);
        return null;
    }

    public Object caseACharValue(CharValue v)
    {
        if(mChar!=null) setIcon(mChar);
        return null;
    }

    public Object caseABoolValue(BoolValue v)
    {
        if(mBool!=null) setIcon(mBool);
        return null;
    }

}