/* *******************************************************************************
 *   Kenya - an introductory programming language and development environment    *
 *   Copyright (C) 2001,2002 Robert Chatley, Alan Chan                           *
 *                                                                               *
 *   This program is free software; you can redistribute it and/or               *
 *   modify it under the terms of the GNU General Public License                 *
 *   as published by the Free Software Foundation; either version 2              *
 *   of the License, or (at your option) any later version.                      *
 *                                                                               *
 *   This program is distributed in the hope that it will be useful,             *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *   GNU General Public License for more details.                                *
 *                                                                               *
 *   You should have received a copy of the GNU General Public License           *
 *   along with this program; if not, write to the Free Software                 *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
 *                                                                               *
 *   The author can be contacted by email at robert@chatley.com                  *
 *                                                                               *
 *********************************************************************************/

/**
 * ArrayDim.java
 *
 * Stores dimension information about an array
 * variable. Used by interpreter to read and
 * write array variables.
 */
 package com.chatley.kenya.interpreter;

 public class ArrayDim
 {
    //private int m1D = -1; // size of first dimension
    //private int m2D = -1; // size of second dimension
    //private int m3D = -1; // size of third dimension

    // for the moment, assume that no arrays are > 5d
    // remember to change this const if we extend later.
    public static final int MAX_SIZE = 5;

    /** The lengths of the individual dimensions.*/
    private int[] lengths = new int[MAX_SIZE];

    /** The number of dimensions.*/
    private int mDims = 0;


    public ArrayDim(){}

    public ArrayDim(int a1D)
    {
        lengths[mDims++] = a1D;
    }

    public ArrayDim(int a1D, int a2D)
    {
        this(a1D);
        lengths[mDims++] = a2D;
    }

    public ArrayDim(int a1D, int a2D, int a3D)
    {
        this(a1D,a2D);
        lengths[mDims++] = a3D;
    }

    public int getDim(int dim)
    {
        if(dim > 0 && dim <= mDims){
            return lengths[dim-1];
        }else{
            throw new ArrayIndexOutOfBoundsException(dim);
        }
    }

    /**
     * Add a dimension to the array. If there are
     * already MAX_SIZE dimensions to this array, subsequent
     * calls to this method are ignored.
     * @param aDim the size of the dimension we wish to add.
     */
    public void addDim(int aDim)
    {
        if(mDims < MAX_SIZE) lengths[mDims++] = aDim;
        // otherwise discard value;
    }

    /** Returns the total capacity of this array.*/
    public int getSize()
    {
        int size = 1;
        for(int i=0;i<mDims;i++) size*= lengths[i];
        return size;
    }

    /** Returns the number of dimensions this array has.*/
    public int getNumDims()
    {
        return mDims;
    }

    /** Returns the size of the dimensions of the array as an array.*/
    public int[] getDims()
    {
        return lengths;
    }
}