/* *******************************************************************************
 *   Kenya - an introductory programming language and development environment    *
 *   Copyright (C) 2001,2002 Robert Chatley, Alan Chan                           *
 *                                                                               *
 *   This program is free software; you can redistribute it and/or               *
 *   modify it under the terms of the GNU General Public License                 *
 *   as published by the Free Software Foundation; either version 2              *
 *   of the License, or (at your option) any later version.                      *
 *                                                                               *
 *   This program is distributed in the hope that it will be useful,             *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *   GNU General Public License for more details.                                *
 *                                                                               *
 *   You should have received a copy of the GNU General Public License           *
 *   along with this program; if not, write to the Free Software                 *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
 *                                                                               *
 *   The author can be contacted by email at robert@chatley.com                  *
 *                                                                               *
 *********************************************************************************/

/**
 * Kenya Interpreter Runtime exception.
 *
 * This is an exception thrown by the kenya interpreter.
 * It subclasses RuntimeException rather than Exception
 * because the interpreter subclasses DepthFirstAdaptor
 * and overrides the vistor methods in that. This prevents
 * the throwing of 'normal' exceptions from these methods
 * because we are not allowed to change the signatures of
 * these vistor methods.
 */


package com.chatley.kenya.interpreter;

public class KInterException extends RuntimeException
{
    private int mLine = -1;

    public KInterException(String msg)
    {
        super("Error: " + msg);
    }

    public KInterException(String msg, int LineNum)
    {
        super("Error (line " + LineNum + "): " + msg);
        mLine = LineNum;
    }

    /**
     * Return the line number of kenya program which caused
     * this exception.
     */
    public int getLine()
    {
        return mLine;
    }
}