/* *******************************************************************************
 *   Kenya - an introductory programming language and development environment    *
 *   Copyright (C) 2001,2002 Robert Chatley, Alan Chan                           *
 *                                                                               *
 *   This program is free software; you can redistribute it and/or               *
 *   modify it under the terms of the GNU General Public License                 *
 *   as published by the Free Software Foundation; either version 2              *
 *   of the License, or (at your option) any later version.                      *
 *                                                                               *
 *   This program is distributed in the hope that it will be useful,             *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *   GNU General Public License for more details.                                *
 *                                                                               *
 *   You should have received a copy of the GNU General Public License           *
 *   along with this program; if not, write to the Free Software                 *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
 *                                                                               *
 *   The author can be contacted by email at robert@chatley.com                  *
 *                                                                               *
 *********************************************************************************/

/*
 * A member of a set of classes which has usage similar to that of a
 * union type in C. See the class Value for more details.
 *
 * This class is used to store the value of a char variable in a Kenya
 * program when running the Kenya interpreter.
 */

package com.chatley.kenya.interpreter.value;

public class CharValue extends PrimitiveValue
{

    private char mChar;

    /**
     * Initialise to hold a char.
     * @param c the char value.
     */
    public CharValue(char c)
    {
        mChar = c;
    }

    /**
     * Initialise to hold a char with a variable name.
     * @param c the char value.
     * @param varName the variable name.
     */
    public CharValue(char c, String varName)
    {
        this(c);
        setVarName(varName);
    }

    /**
     * returns the value of this object as a char.
     */
    public char getChar()
    {
        return mChar;
    }

    /**
     * Sets this object to represent a new char value.
     * @param c the new char value.
     */
    public void setChar(char c)
    {
        mChar = c;
    }

    /**
     * Returns the value of this object as a string.
     */
    public String toString()
    {
        return String.valueOf(mChar);
    }

    /**
     * Reports the type of this object as "String".
     */
    public String getType()
    {
        return "char";
    }

    public boolean isChar()
    {
        return true;
    }

    public Object apply(ValueSwitch vs)
    {
        return vs.caseACharValue(this);
    }
}