/* *******************************************************************************
 *   Kenya - an introductory programming language and development environment    *
 *   Copyright (C) 2001,2002 Robert Chatley, Alan Chan                           *
 *                                                                               *
 *   This program is free software; you can redistribute it and/or               *
 *   modify it under the terms of the GNU General Public License                 *
 *   as published by the Free Software Foundation; either version 2              *
 *   of the License, or (at your option) any later version.                      *
 *                                                                               *
 *   This program is distributed in the hope that it will be useful,             *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *   GNU General Public License for more details.                                *
 *                                                                               *
 *   You should have received a copy of the GNU General Public License           *
 *   along with this program; if not, write to the Free Software                 *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
 *                                                                               *
 *   The author can be contacted by email at robert@chatley.com                  *
 *                                                                               *
 *********************************************************************************/

/*
 * A member of a set of classes which has usage similar to that of a
 * union type in C. See the class Value for more details.
 *
 * This class is used to store the value of a double variable in a Kenya
 * program when running the Kenya interpreter.
 */

package com.chatley.kenya.interpreter.value;

public class DoubleValue extends NumberValue
{

    private double mDouble;


    /** Initialise to hold a double. */
    public DoubleValue(double d)
    {
        mDouble = d;
    }

    public DoubleValue(double d, String varName)
    {
        this(d);
        setVarName(varName);
    }

    /**
     * returns the value of this Value object as a double.
     */
    public double getNumber()
    {
        return mDouble;
    }

    /**
     * returns the value of this Value object as a int.
     */
    public int getInt()
    {
        return ((int)mDouble);
    }

    /**
     * returns the value of this Value object as a double.
     */
    public double getDouble()
    {
        return mDouble;
    }

    /**
     * Sets this Value object to represent a new double value.
     * @param d the new double value.
     */
    public void setDouble(double d)
    {
        mDouble = d;
    }

    /**
     * Returns the value of this object as a string.
     * If the object does not hold one of the standard
     * Kenya data types, the string "null" is returned.
     */
    public String toString()
    {
        return String.valueOf(mDouble);
    }

    /**
     * Returns the type of this object as a string.
     * If the object does not hold one of the standard
     * Kenya data types, the string "null" is returned.
     */
    public String getType()
    {
        return "double";
    }

    public boolean isDouble()
    {
        return true;
    }

    public Object apply(ValueSwitch vs)
    {
        return vs.caseADoubleValue(this);
    }
}