/* *******************************************************************************
 *   Kenya - an introductory programming language and development environment    *
 *   Copyright (C) 2001,2002 Robert Chatley, Alan Chan                           *
 *                                                                               *
 *   This program is free software; you can redistribute it and/or               *
 *   modify it under the terms of the GNU General Public License                 *
 *   as published by the Free Software Foundation; either version 2              *
 *   of the License, or (at your option) any later version.                      *
 *                                                                               *
 *   This program is distributed in the hope that it will be useful,             *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *   GNU General Public License for more details.                                *
 *                                                                               *
 *   You should have received a copy of the GNU General Public License           *
 *   along with this program; if not, write to the Free Software                 *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
 *                                                                               *
 *   The author can be contacted by email at robert@chatley.com                  *
 *                                                                               *
 *********************************************************************************/

/*
 * A member of a set of classes which has usage similar to that of a
 * union type in C. See the class Value for more details.
 *
 * This class is used to store the value of a integer variable in a Kenya
 * program when running the Kenya interpreter.
 */

package com.chatley.kenya.interpreter.value;

public class IntValue extends NumberValue
{

    private int mInt;


    /**
     * Initialise to hold an integer.
     * @param i the integer value.
     */
    public IntValue(int i)
    {
        mInt = i;
    }

    /**
     * Initialise to hold an integer with a variable name.
     * @param i the integer value.
     * @param varName the variable name.
     */
    public IntValue(int i, String varName)
    {
        this(i);
        setVarName(varName);
    }

    /**
     * returns the value of this object as a double.
     */
    public double getNumber()
    {
        return (double)mInt;
    }

    /**
     * returns the value of this object as a int.
     */
    public int getInt()
    {
        return mInt;
    }

    /**
     * returns the value of this object as a double.
     */
    public double getDouble()
    {
        return getNumber();
    }

    /**
     * Sets this object to represent a new int value.
     * @param i the new int value.
     */
    public void setInt(int i)
    {
        mInt = i;
    }

    /**
     * Returns the value of this object as a string.
     */
    public String toString()
    {
        return String.valueOf(mInt);
    }

    /**
     * Reports the type of this object as "int".
     */
    public String getType()
    {
        return "int";
    }

    public boolean isInt()
    {
        return true;
    }

    public Object apply(ValueSwitch vs)
    {
        return vs.caseAIntValue(this);
    }
}