/* *******************************************************************************
 *   Kenya - an introductory programming language and development environment    *
 *   Copyright (C) 2001,2002 Robert Chatley, Alan Chan                           *
 *                                                                               *
 *   This program is free software; you can redistribute it and/or               *
 *   modify it under the terms of the GNU General Public License                 *
 *   as published by the Free Software Foundation; either version 2              *
 *   of the License, or (at your option) any later version.                      *
 *                                                                               *
 *   This program is distributed in the hope that it will be useful,             *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *   GNU General Public License for more details.                                *
 *                                                                               *
 *   You should have received a copy of the GNU General Public License           *
 *   along with this program; if not, write to the Free Software                 *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
 *                                                                               *
 *   The author can be contacted by email at robert@chatley.com                  *
 *                                                                               *
 *********************************************************************************/

/*
 * A member of a set of classes which has usage similar to that of a
 * union type in C. See the class Value for more details.
 *
 * This class is used to represent null in a Kenya
 * program when running the Kenya interpreter.
 */

package com.chatley.kenya.interpreter.value;

public class NullValue extends ReferenceValue
{

    /** The type of reference that this should be (??) */
    private String mType;

    /**
     * Creates a new NullValue object with no additional information.
     */
    public NullValue(){}

    /**
     * Initialise to be a null of a certain type.
     * @param type the reference type this is a null of.
     */
    public NullValue(String type)
    {
        mType = type;
    }

    /**
     * Initialise to be a null of a certain type with a variable name.
     * @param type the reference type this is a null of.
     * @param varName the variable name.
     */
    public NullValue(String type, String varName)
    {
        this(type);
        setVarName(varName);
    }

    /**
     * Returns the value of this object as a string.
     */
    public String toString()
    {
        return "null";
    }

    /**
     * Reports the type of this object as "int".
     */
    public String getType()
    {
        return mType;
    }

    public boolean isNull()
    {
        return true;
    }

    public Object apply(ValueSwitch vs)
    {
        return vs.caseANullValue(this);
    }

    public Object clone() throws CloneNotSupportedException
    {
        return super.clone();
    }
}