/* *******************************************************************************
 *   Kenya - an introductory programming language and development environment    *
 *   Copyright (C) 2001,2002 Robert Chatley, Alan Chan                           *
 *                                                                               *
 *   This program is free software; you can redistribute it and/or               *
 *   modify it under the terms of the GNU General Public License                 *
 *   as published by the Free Software Foundation; either version 2              *
 *   of the License, or (at your option) any later version.                      *
 *                                                                               *
 *   This program is distributed in the hope that it will be useful,             *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *   GNU General Public License for more details.                                *
 *                                                                               *
 *   You should have received a copy of the GNU General Public License           *
 *   along with this program; if not, write to the Free Software                 *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
 *                                                                               *
 *   The author can be contacted by email at robert@chatley.com                  *
 *                                                                               *
 *********************************************************************************/

/*
 * A member of a set of classes which has usage similar to that of a
 * union type in C. See the class Value for more details.
 *
 * This class is used to store the value of a string variable in a Kenya
 * program when running the Kenya interpreter.
 *
 * Variables of type String in a kenya program like <code>String s</code> will
 * be represented using instances of this class.
 *
 */

package com.chatley.kenya.interpreter.value;

public class StringValue extends PrimitiveValue
{

    private String mString;

    /**
     * Initialise to hold a string.
     * @param s the String value.
     */
    public StringValue(String s)
    {
        mString = s;
    }

    /**
     * Initialise to hold a string with a variable name.
     * @param s the String value.
     * @param varName the variable name.
     */
    public StringValue(String s, String varName)
    {
        this(s);
        setVarName(varName);
    }

    /**
     * returns the value of this object as a String.
     */
    public String getString()
    {
        return mString;
    }

    /**
     * Sets this object to represent a new String value.
     * @param s the new String value.
     */
    public void setString(String s)
    {
        mString = s;
    }

    /**
     * Returns the value of this object as a string.
     */
    public String toString()
    {
        return getString();
    }

    /**
     * Reports the type of this object as "String".
     */
    public String getType()
    {
        return "String";
    }

    public boolean isString()
    {
        return true;
    }

    public Object apply(ValueSwitch vs)
    {
        return vs.caseAStringValue(this);
    }

    public Object clone() throws CloneNotSupportedException
    {
        return (Object)
          new StringValue(new String(mString),new String(getVarName()));
    }
}