/* *******************************************************************************
 *   Kenya - an introductory programming language and development environment    *
 *   Copyright (C) 2001,2002 Robert Chatley, Alan Chan                           *
 *                                                                               *
 *   This program is free software; you can redistribute it and/or               *
 *   modify it under the terms of the GNU General Public License                 *
 *   as published by the Free Software Foundation; either version 2              *
 *   of the License, or (at your option) any later version.                      *
 *                                                                               *
 *   This program is distributed in the hope that it will be useful,             *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *   GNU General Public License for more details.                                *
 *                                                                               *
 *   You should have received a copy of the GNU General Public License           *
 *   along with this program; if not, write to the Free Software                 *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
 *                                                                               *
 *   The author can be contacted by email at robert@chatley.com                  *
 *                                                                               *
 *********************************************************************************/

/**
 * FuncInfo.java
 *
 * Class which groups together information about a function
 * declaration. Used by Kenya interpreter.
 */
//package kenya.interpreter;
package com.chatley.kenya.utils;

import java.util.*;
import com.chatley.kenya.minijava.node.*;

public class FuncInfo
{
    /** Name of function.*/
    String mName;

    /** Return type of function.*/
    String mType;

    /** Point of entry of function on AST.*/
    Node mEntryPt;

    // Note: the following vectors should have equal sizes.

    /** Names of parameters, if any.*/
    Vector mParamNames;

    /** Types of parameters, if any.*/
    Vector mParamTypes;

    public FuncInfo(String aName, String aType, Node aEntry)
    {
        mName = aName;
        mType = aType;
        mEntryPt = aEntry;
    }

    /** Returns the number of params*/
    public int numParams()
    {
        if(mParamNames==null) return 0;
        else return mParamNames.size();
    }

    public void addParam(String aName, String aType)
    {
        if(mParamNames==null){
            mParamNames = new Vector();
            mParamTypes = new Vector();
        }
        mParamNames.add(aName);
        mParamTypes.add(aType);
    }

    public String getParam(int index)
    {
        return (String)mParamNames.elementAt(index);
    }

    public String getParamType(int index)
    {
        return (String)mParamTypes.elementAt(index);
    }

    public Node getEntryPt()
    {
        return mEntryPt;
    }

    public String getName()
    {
        return mName;
    }

    public String getType()
    {
        return mType;
    }

    public String toString()
    {
        String s;
        s = mType + " " + mName + " (";
        for(int i=0;i<numParams();i++){
            if(i!=0) s+= ", ";
            s+= getParamType(i) + " " + getParam(i);
        }
        s+= ")";
        return s;
    }

}