/* *******************************************************************************
 *   Kenya - an introductory programming language and development environment    *
 *   Copyright (C) 2001,2002 Robert Chatley, Alan Chan                           *
 *                                                                               *
 *   This program is free software; you can redistribute it and/or               *
 *   modify it under the terms of the GNU General Public License                 *
 *   as published by the Free Software Foundation; either version 2              *
 *   of the License, or (at your option) any later version.                      *
 *                                                                               *
 *   This program is distributed in the hope that it will be useful,             *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of              *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               *
 *   GNU General Public License for more details.                                *
 *                                                                               *
 *   You should have received a copy of the GNU General Public License           *
 *   along with this program; if not, write to the Free Software                 *
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. *
 *                                                                               *
 *   The author can be contacted by email at robert@chatley.com                  *
 *                                                                               *
 *********************************************************************************/

/**
 * class that provides a single point of control for debug messages to console,
 * intended as a development aid only.
 *
 * Alan Chan 2002/7
 */
package com.chatley.kenya.utils;

import java.util.Date;
import java.text.DateFormat;
import java.text.SimpleDateFormat;

public class IMTrace
{
  public static final int DETAIL = 1;
  public static final int INFO = 2;
  public static final int PROGRESS = 3;
  public static final int WARNING = 4;
  public static final int ERROR = 5;
  public static final int ALLOFF = 6;

  private static int mTraceLevel  = INFO;

  // debug mode is off by default
  private static boolean mDebugOn = false;

  private static DateFormat mTimestampFormat = new SimpleDateFormat("HH:mm:ss");

  private IMTrace(){}

  public static void setTraceLevel( int aTraceLevel )
  {
    mTraceLevel = aTraceLevel;
  }

  public static int getTraceLevel()
  {
    return mTraceLevel;
  }

  public static void setDebugOn()
  {
    mDebugOn = true;
  }

  public static void setDebugOff()
  {
    mDebugOn = false;
  }

  public static void out( String aJavaString )
  {
    out( INFO, aJavaString );
  }

  public static void out( int aTraceLevel, String aJavaString )
  {
    // if debugging is off, don't print anything
    if(!mDebugOn) return;

    // setting the tracelevel will filter out all messages which are of a lower
    // level
    if (aTraceLevel >= mTraceLevel)
    {
      printTraceLevel(aTraceLevel);
      System.out.println( aJavaString );
    }
  }

  private static void printTraceLevel(int aTraceLevel)
  {
      switch (aTraceLevel)
      {
        case DETAIL: System.out.print( getTime() + " : DETAIL:\t" ); break;
        case INFO: System.out.print( getTime() + " : INFO :\t\t" ); break;
        case PROGRESS: System.out.print( getTime() + " : PROGRESS:\t" ); break;
        case WARNING: System.out.print( getTime() + " : WARNING:\t" ); break;
        case ERROR: System.out.print( getTime() + " : ERROR:\t\t" ); break;
      }
  }

  // will return a string for the current time, like "21:53:01"
  private static String getTime()
  {
      java.util.Date now = new java.util.Date();
      return mTimestampFormat.format(now).toString();
  }
}

